import os
import numpy as np
from joblib import dump, load

from ASAPPpy import ROOT_PATH
from .feature_engineering.lexical_features import create_word_ngrams, create_multiple_word_ngrams, create_character_ngrams, create_multiple_character_ngrams, compute_jaccard, compute_dice, compute_overlap, NG
from .feature_engineering.syntactic_features import compute_pos, dependency_parsing
from .feature_engineering.semantic_features import compute_ner, compute_semantic_relations
from .feature_selection.feature_selection import feature_selection
from .models.word2vec.word2vec import word2vec_model
from .models.fastText.fasttext import fasttext_model
from .models.ontoPT.ptlkb import ptlkb_model
# TODO: rename and relocate this file:
from .load_embeddings import word_embeddings_model
from .scripts.tools import build_sentences_from_tokens, compute_tfidf_matrix, preprocessing

from NLPyPort.FullPipeline import new_full_pipe

class STSModel():
    '''Semantic Textual Similarity Model.

    Add longer class information here....

    Attributes:
        model_name: .
        model: .
        number_features: .
        used_features: .
        lexical_features: .
        syntactic_features: .
        semantic_features: .
        distributional_features: .
        all_features: .
        feature_selection: .
    '''

    def __init__(self, model_name='default_model_name', model=None, number_features=0):
        self.model_name = model_name
        self.model = model
        self.number_features = number_features
        self.used_features = {}
        self.lexical_features = None
        self.syntactic_features = None
        self.semantic_features = None
        self.distributional_features = None
        self.all_features = None
        self.feature_selection = 0

    # TODO (TESTING): there is no option to load a model that uses feature selection.
    def extract_lexical_features(self, corpus, wn_jaccard, wn_dice, wn_overlap, cn_jaccard, cn_dice, cn_overlap):
        '''
        Parameters
        ----------


        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return
        else:
            preprocessed_corpus = preprocessing(corpus, 0, 0, 0, 0)

        # create word ngrams of different sizes
        if wn_jaccard or wn_dice or wn_overlap:
            word_ngrams_1 = create_word_ngrams(preprocessed_corpus, 1)
            word_ngrams_2 = create_word_ngrams(preprocessed_corpus, 2)
            word_ngrams_3 = create_word_ngrams(preprocessed_corpus, 3)

        # compute distance coefficients for these ngrams
        if wn_jaccard:
            wn_jaccard_1 = compute_jaccard(word_ngrams_1)
            wn_jaccard_2 = compute_jaccard(word_ngrams_2)
            wn_jaccard_3 = compute_jaccard(word_ngrams_3)
            self.used_features.update({'wn_jaccard_1': 1, 'wn_jaccard_2': 1, 'wn_jaccard_3': 1})
        else:
            self.used_features.update({'wn_jaccard_1': 0, 'wn_jaccard_2': 0, 'wn_jaccard_3': 0})

        if wn_dice:
            wn_dice_1 = compute_dice(word_ngrams_1)
            wn_dice_2 = compute_dice(word_ngrams_2)
            wn_dice_3 = compute_dice(word_ngrams_3)
            self.used_features.update({'wn_dice_1': 1, 'wn_dice_2': 1, 'wn_dice_3': 1})
        else:
            self.used_features.update({'wn_dice_1': 0, 'wn_dice_2': 0, 'wn_dice_3': 0})

        if wn_overlap:
            wn_overlap_1 = compute_overlap(word_ngrams_1)
            wn_overlap_2 = compute_overlap(word_ngrams_2)
            wn_overlap_3 = compute_overlap(word_ngrams_3)
            self.used_features.update({'wn_overlap_1': 1, 'wn_overlap_2': 1, 'wn_overlap_3': 1})
        else:
            self.used_features.update({'wn_overlap_1': 0, 'wn_overlap_2': 0, 'wn_overlap_3': 0})

        # create character ngrams of different sizes
        if cn_jaccard or cn_dice or cn_overlap:
            character_ngrams_2 = create_character_ngrams(preprocessed_corpus, 2)
            character_ngrams_3 = create_character_ngrams(preprocessed_corpus, 3)
            character_ngrams_4 = create_character_ngrams(preprocessed_corpus, 4)

        # compute distance coefficients for these ngrams
        if cn_jaccard:
            cn_jaccard_2 = compute_jaccard(character_ngrams_2)
            cn_jaccard_3 = compute_jaccard(character_ngrams_3)
            cn_jaccard_4 = compute_jaccard(character_ngrams_4)
            self.used_features.update({'cn_jaccard_2': 1, 'cn_jaccard_3': 1, 'cn_jaccard_4': 1})
        else:
            self.used_features.update({'cn_jaccard_2': 0, 'cn_jaccard_3': 0, 'cn_jaccard_4': 0})

        if cn_dice:
            cn_dice_2 = compute_dice(character_ngrams_2)
            cn_dice_3 = compute_dice(character_ngrams_3)
            cn_dice_4 = compute_dice(character_ngrams_4)
            self.used_features.update({'cn_dice_2': 1, 'cn_dice_3': 1, 'cn_dice_4': 1})
        else:
            self.used_features.update({'cn_dice_2': 0, 'cn_dice_3': 0, 'cn_dice_4': 0})

        if cn_overlap:
            cn_overlap_2 = compute_overlap(character_ngrams_2)
            cn_overlap_3 = compute_overlap(character_ngrams_3)
            cn_overlap_4 = compute_overlap(character_ngrams_4)
            self.used_features.update({'cn_overlap_2': 1, 'cn_overlap_3': 1, 'cn_overlap_4': 1})
        else:
            self.used_features.update({'cn_overlap_2': 0, 'cn_overlap_3': 0, 'cn_overlap_4': 0})

        tmp_lexical_features = []

        for pair in range(len(corpus)//2):
            pair_features = []

            if wn_jaccard:
                pair_features.append(wn_jaccard_1['jaccard'][pair])
                pair_features.append(wn_jaccard_2['jaccard'][pair])
                pair_features.append(wn_jaccard_3['jaccard'][pair])
            else:
                pair_features.extend([0]*3)

            if wn_dice:
                pair_features.append(wn_dice_1['dice'][pair])
                pair_features.append(wn_dice_2['dice'][pair])
                pair_features.append(wn_dice_3['dice'][pair])
            else:
                pair_features.extend([0]*3)

            if wn_overlap:
                pair_features.append(wn_overlap_1['overlap'][pair])
                pair_features.append(wn_overlap_2['overlap'][pair])
                pair_features.append(wn_overlap_3['overlap'][pair])
            else:
                pair_features.extend([0]*3)

            if cn_jaccard:
                pair_features.append(cn_jaccard_2['jaccard'][pair])
                pair_features.append(cn_jaccard_3['jaccard'][pair])
                pair_features.append(cn_jaccard_4['jaccard'][pair])
            else:
                pair_features.extend([0]*3)

            if cn_dice:
                pair_features.append(cn_dice_2['dice'][pair])
                pair_features.append(cn_dice_3['dice'][pair])
                pair_features.append(cn_dice_4['dice'][pair])
            else:
                pair_features.extend([0]*3)

            if cn_overlap:
                pair_features.append(cn_overlap_2['overlap'][pair])
                pair_features.append(cn_overlap_3['overlap'][pair])
                pair_features.append(cn_overlap_4['overlap'][pair])
            else:
                pair_features.extend([0]*3)

            pair_features_tuple = tuple(pair_features)
            tmp_lexical_features.append(pair_features_tuple)

        self.lexical_features = np.array(tmp_lexical_features)

        return self.lexical_features

    def _extract_lexical_features(self, corpus):
        '''
        Parameters
        ----------
        When loading a previously trained model this function extracts only the features that were used by it instead of extracting them all.

        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return
        else:
            preprocessed_corpus = preprocessing(corpus, 0, 0, 0, 0)

        # check if any wn feature was used by the model before computing the word n_grams
        if any(key[:2] == 'wn' and value == 1 for key, value in self.used_features.items()):
            word_ngrams_1 = create_word_ngrams(preprocessed_corpus, 1)
            word_ngrams_2 = create_word_ngrams(preprocessed_corpus, 2)
            word_ngrams_3 = create_word_ngrams(preprocessed_corpus, 3)

        # compute distance coefficients for these ngrams
        if self.used_features['wn_jaccard_1'] == 1:
            wn_jaccard_1 = compute_jaccard(word_ngrams_1)
        if self.used_features['wn_jaccard_2'] == 1:
            wn_jaccard_2 = compute_jaccard(word_ngrams_2)
        if self.used_features['wn_jaccard_3'] == 1:
            wn_jaccard_3 = compute_jaccard(word_ngrams_3)

        if self.used_features['wn_dice_1'] == 1:
            wn_dice_1 = compute_dice(word_ngrams_1)
        if self.used_features['wn_dice_2'] == 1:
            wn_dice_2 = compute_dice(word_ngrams_2)
        if self.used_features['wn_dice_3'] == 1:
            wn_dice_3 = compute_dice(word_ngrams_3)

        if self.used_features['wn_overlap_1'] == 1:
            wn_overlap_1 = compute_overlap(word_ngrams_1)
        if self.used_features['wn_overlap_2'] == 1:
            wn_overlap_2 = compute_overlap(word_ngrams_2)
        if self.used_features['wn_overlap_3'] == 1:
            wn_overlap_3 = compute_overlap(word_ngrams_3)

        # check if any cn feature was used by the model before computing the character n_grams
        if any(key[:2] == 'cn' and value == 1 for key, value in self.used_features.items()):
            character_ngrams_2 = create_character_ngrams(preprocessed_corpus, 2)
            character_ngrams_3 = create_character_ngrams(preprocessed_corpus, 3)
            character_ngrams_4 = create_character_ngrams(preprocessed_corpus, 4)

        # compute distance coefficients for these ngrams
        if self.used_features['cn_jaccard_2'] == 1:
            cn_jaccard_2 = compute_jaccard(character_ngrams_2)
        if self.used_features['cn_jaccard_3'] == 1:
            cn_jaccard_3 = compute_jaccard(character_ngrams_3)
        if self.used_features['cn_jaccard_4'] == 1:
            cn_jaccard_4 = compute_jaccard(character_ngrams_4)

        if self.used_features['cn_dice_2'] == 1:
            cn_dice_2 = compute_dice(character_ngrams_2)
        if self.used_features['cn_dice_3'] == 1:
            cn_dice_3 = compute_dice(character_ngrams_3)
        if self.used_features['cn_dice_4'] == 1:
            cn_dice_4 = compute_dice(character_ngrams_4)

        if self.used_features['cn_overlap_2'] == 1:
            cn_overlap_2 = compute_overlap(character_ngrams_2)
        if self.used_features['cn_overlap_3'] == 1:
            cn_overlap_3 = compute_overlap(character_ngrams_3)
        if self.used_features['cn_overlap_4'] == 1:
            cn_overlap_4 = compute_overlap(character_ngrams_4)

        tmp_lexical_features = []

        for pair in range(len(corpus)//2):
            pair_features = []

            if self.used_features['wn_jaccard_1'] == 1:
                pair_features.append(wn_jaccard_1['jaccard'][pair])
            if self.used_features['wn_jaccard_2'] == 1:
                pair_features.append(wn_jaccard_2['jaccard'][pair])
            if self.used_features['wn_jaccard_3'] == 1:
                pair_features.append(wn_jaccard_3['jaccard'][pair])

            if self.used_features['wn_dice_1'] == 1:
                pair_features.append(wn_dice_1['dice'][pair])
            if self.used_features['wn_dice_2'] == 1:
                pair_features.append(wn_dice_2['dice'][pair])
            if self.used_features['wn_dice_3'] == 1:
                pair_features.append(wn_dice_3['dice'][pair])

            if self.used_features['wn_overlap_1'] == 1:
                pair_features.append(wn_overlap_1['overlap'][pair])
            if self.used_features['wn_overlap_2'] == 1:
                pair_features.append(wn_overlap_2['overlap'][pair])
            if self.used_features['wn_overlap_3'] == 1:
                pair_features.append(wn_overlap_3['overlap'][pair])

            if self.used_features['cn_jaccard_2'] == 1:
                pair_features.append(cn_jaccard_2['jaccard'][pair])
            if self.used_features['cn_jaccard_3'] == 1:
                pair_features.append(cn_jaccard_3['jaccard'][pair])
            if self.used_features['cn_jaccard_4'] == 1:
                pair_features.append(cn_jaccard_4['jaccard'][pair])

            if self.used_features['cn_dice_2'] == 1:
                pair_features.append(cn_dice_2['dice'][pair])
            if self.used_features['cn_dice_3'] == 1:
                pair_features.append(cn_dice_3['dice'][pair])
            if self.used_features['cn_dice_4'] == 1:
                pair_features.append(cn_dice_4['dice'][pair])

            if self.used_features['cn_overlap_2'] == 1:
                pair_features.append(cn_overlap_2['jaccard'][pair])
            if self.used_features['cn_overlap_3'] == 1:
                pair_features.append(cn_overlap_3['jaccard'][pair])
            if self.used_features['cn_overlap_4'] == 1:
                pair_features.append(cn_overlap_4['jaccard'][pair])

            pair_features_tuple = tuple(pair_features)
            tmp_lexical_features.append(pair_features_tuple)

        self.lexical_features = np.array(tmp_lexical_features)

        return self.lexical_features

    def extract_syntactic_features(self, corpus, pos_tags, dependencies):
        '''
        Parameters
        ----------


        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return

        if pos_tags:
            pipeline_tags = new_full_pipe(corpus, options={"pos_tagger":True, "string_or_array":True})
            tags = build_sentences_from_tokens(pipeline_tags.pos_tags)

            # compute POS tags
            pos_tags = compute_pos(tags)
            self.used_features.update({'pos_adj': 1, 'pos_adv': 1, 'pos_art': 1, 'pos_conj-c': 1, 'pos_conj-s': 1, 'pos_intj': 1, 'pos_n': 1, 'pos_n-adj': 1, 'pos_num': 1, 'pos_pron-det': 1, 'pos_pron-indp': 1, 'pos_pron-pers': 1, 'pos_prop': 1, 'pos_prp': 1, 'pos_punc': 1, 'pos_v-fin': 1, 'pos_v-ger': 1, 'pos_v-inf': 1, 'pos_v-pcp': 1})
        else:
            self.used_features.update({'pos_adj': 0, 'pos_adv': 0, 'pos_art': 0, 'pos_conj-c': 0, 'pos_conj-s': 0, 'pos_intj': 0, 'pos_n': 0, 'pos_n-adj': 0, 'pos_num': 0, 'pos_pron-det': 0, 'pos_pron-indp': 0, 'pos_pron-pers': 0, 'pos_prop': 0, 'pos_prp': 0, 'pos_punc': 0, 'pos_v-fin': 0, 'pos_v-ger': 0, 'pos_v-inf': 0, 'pos_v-pcp': 0})

        if dependencies:
            # compute Syntactic Dependency parsing
            dependencies = dependency_parsing(corpus)
            self.used_features.update({'dependency_parsing': 1})
        else:
            self.used_features.update({'dependency_parsing': 0})

        tmp_syntactic_features = []

        for pair in range(len(corpus)//2):
            pair_features = []

            if not pos_tags.empty:
                pair_features.append(pos_tags['adj'][pair])
                pair_features.append(pos_tags['adv'][pair])
                pair_features.append(pos_tags['art'][pair])
                pair_features.append(pos_tags['conj-c'][pair])
                pair_features.append(pos_tags['conj-s'][pair])
                pair_features.append(pos_tags['intj'][pair])
                pair_features.append(pos_tags['n'][pair])
                pair_features.append(pos_tags['n-adj'][pair])
                pair_features.append(pos_tags['num'][pair])
                pair_features.append(pos_tags['pron-det'][pair])
                pair_features.append(pos_tags['pron-indp'][pair])
                pair_features.append(pos_tags['pron-pers'][pair])
                pair_features.append(pos_tags['prop'][pair])
                pair_features.append(pos_tags['prp'][pair])
                pair_features.append(pos_tags['punc'][pair])
                pair_features.append(pos_tags['v-fin'][pair])
                pair_features.append(pos_tags['v-ger'][pair])
                pair_features.append(pos_tags['v-inf'][pair])
                pair_features.append(pos_tags['v-pcp'][pair])
            else:
                pair_features.extend([0]*19)

            if not dependencies.empty:
                pair_features.append(dependencies['dependency_parsing_jc'][pair])
            else:
                pair_features.extend([0]*1)

            pair_features_tuple = tuple(pair_features)
            tmp_syntactic_features.append(pair_features_tuple)

        self.syntactic_features = np.array(tmp_syntactic_features)

        return self.syntactic_features

    def _extract_syntactic_features(self, corpus):
        '''
        Parameters
        ----------


        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return

        # check if any pos feature was used by the model before computing the pipeline tags and respective features
        if any(key[:3] == 'pos' and value == 1 for key, value in self.used_features.items()):
            pipeline_tags = new_full_pipe(corpus, options={"pos_tagger":True, "string_or_array":True})
            tags = build_sentences_from_tokens(pipeline_tags.pos_tags)

            # compute POS tags
            pos_tags = compute_pos(tags)
        else:
            pos_tags = None

        if self.used_features['dependency_parsing'] == 1:
            # compute Syntactic Dependency parsing
            dependencies = dependency_parsing(corpus)
        else:
            dependencies = None

        if pos_tags is None and dependencies is None:
            return None
        else:
            tmp_syntactic_features = []

            for pair in range(len(corpus)//2):
                pair_features = []

                if pos_tags is not None:
                    if not pos_tags.empty:
                        if self.used_features['pos_adj'] == 1:
                            pair_features.append(pos_tags['adj'][pair])
                        if self.used_features['pos_adv'] == 1:
                            pair_features.append(pos_tags['adv'][pair])
                        if self.used_features['pos_art'] == 1:
                            pair_features.append(pos_tags['art'][pair])
                        if self.used_features['pos_conj-c'] == 1:
                            pair_features.append(pos_tags['conj-c'][pair])
                        if self.used_features['pos_conj-s'] == 1:
                            pair_features.append(pos_tags['conj-s'][pair])
                        if self.used_features['pos_intj'] == 1:
                            pair_features.append(pos_tags['intj'][pair])
                        if self.used_features['pos_n'] == 1:
                            pair_features.append(pos_tags['n'][pair])
                        if self.used_features['pos_n-adj'] == 1:
                            pair_features.append(pos_tags['n-adj'][pair])
                        if self.used_features['pos_num'] == 1:
                            pair_features.append(pos_tags['num'][pair])
                        if self.used_features['pos_pron-det'] == 1:
                            pair_features.append(pos_tags['pron-det'][pair])
                        if self.used_features['pos_pron-indp'] == 1:
                            pair_features.append(pos_tags['pron-indp'][pair])
                        if self.used_features['pos_pron-pers'] == 1:
                            pair_features.append(pos_tags['pron-pers'][pair])
                        if self.used_features['pos_prop'] == 1:
                            pair_features.append(pos_tags['prop'][pair])
                        if self.used_features['pos_prp'] == 1:
                            pair_features.append(pos_tags['prp'][pair])
                        if self.used_features['pos_punc'] == 1:
                            pair_features.append(pos_tags['punc'][pair])
                        if self.used_features['pos_v-fin'] == 1:
                            pair_features.append(pos_tags['v-fin'][pair])
                        if self.used_features['pos_v-ger'] == 1:
                            pair_features.append(pos_tags['v-ger'][pair])
                        if self.used_features['pos_v-inf'] == 1:
                            pair_features.append(pos_tags['v-inf'][pair])
                        if self.used_features['pos_v-pcp'] == 1:
                            pair_features.append(pos_tags['v-pcp'][pair])

                if dependencies is not None:
                    if not dependencies.empty:
                        if self.used_features['dependency_parsing'] == 1:
                            pair_features.append(dependencies['dependency_parsing_jc'][pair])

                pair_features_tuple = tuple(pair_features)
                tmp_syntactic_features.append(pair_features_tuple)

            self.syntactic_features = np.array(tmp_syntactic_features)

            return self.syntactic_features

    def extract_semantic_features(self, corpus, semantic_relations, ners):
        '''
        Parameters
        ----------


        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return

        if semantic_relations:
            pipeline_lemmas = new_full_pipe(corpus, options={"lemmatizer":True, "string_or_array":True})
            lemmas = build_sentences_from_tokens(pipeline_lemmas.lemas)
            # compute semantic relations coefficients
            semantic_relations = compute_semantic_relations(lemmas)
            self.used_features.update({'sr_antonyms': 1, 'sr_synonyms': 1, 'sr_hyperonyms': 1, 'sr_other': 1})
        else:
            self.used_features.update({'sr_antonyms': 0, 'sr_synonyms': 0, 'sr_hyperonyms': 0, 'sr_other': 0})

        if ners:
            pipeline_entities = new_full_pipe(corpus, options={"entity_recognition":True, "string_or_array":True})
            entities = build_sentences_from_tokens(pipeline_entities.entities)
            # compute NERs
            ners = compute_ner(entities)
            self.used_features.update({'all_ne': 1, 'ne_B-ABSTRACCAO': 1, 'ne_B-ACONTECIMENTO': 1, 'ne_B-COISA': 1, 'ne_B-LOCAL': 1, 'ne_B-OBRA': 1, 'ne_B-ORGANIZACAO': 1, 'ne_B-OUTRO': 1, 'ne_B-PESSOA': 1, 'ne_B-TEMPO': 1, 'ne_B-VALOR': 1})
        else:
            self.used_features.update({'all_ne': 0, 'ne_B-ABSTRACCAO': 0, 'ne_B-ACONTECIMENTO': 0, 'ne_B-COISA': 0, 'ne_B-LOCAL': 0, 'ne_B-OBRA': 0, 'ne_B-ORGANIZACAO': 0, 'ne_B-OUTRO': 0, 'ne_B-PESSOA': 0, 'ne_B-TEMPO': 0, 'ne_B-VALOR': 0})

        tmp_semantic_features = []

        for pair in range(len(corpus)//2):
            pair_features = []

            if not semantic_relations.empty:
                pair_features.append(semantic_relations['antonyms'][pair])
                pair_features.append(semantic_relations['synonyms'][pair])
                pair_features.append(semantic_relations['hyperonyms'][pair])
                pair_features.append(semantic_relations['other'][pair])
            else:
                pair_features.extend([0]*4)

            if not ners.empty:
                pair_features.append(ners['all_ners'][pair])
                pair_features.append(ners['B-ABSTRACCAO'][pair])
                pair_features.append(ners['B-ACONTECIMENTO'][pair])
                pair_features.append(ners['B-COISA'][pair])
                pair_features.append(ners['B-LOCAL'][pair])
                pair_features.append(ners['B-OBRA'][pair])
                pair_features.append(ners['B-ORGANIZACAO'][pair])
                pair_features.append(ners['B-OUTRO'][pair])
                pair_features.append(ners['B-PESSOA'][pair])
                pair_features.append(ners['B-TEMPO'][pair])
                pair_features.append(ners['B-VALOR'][pair])
            else:
                pair_features.extend([0]*11)

            pair_features_tuple = tuple(pair_features)
            tmp_semantic_features.append(pair_features_tuple)

        self.semantic_features = np.array(tmp_semantic_features)

        return self.semantic_features

    def _extract_semantic_features(self, corpus):
        '''
        Parameters
        ----------


        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return

        # check if any sr feature was used by the model before computing the pipeline lemmas and semantic relations features
        if any(key[:2] == 'sr' and value == 1 for key, value in self.used_features.items()):
            pipeline_lemmas = new_full_pipe(corpus, options={"lemmatizer":True, "string_or_array":True})
            lemmas = build_sentences_from_tokens(pipeline_lemmas.lemas)
            # compute semantic relations coefficients
            semantic_relations = compute_semantic_relations(lemmas)
        else:
            semantic_relations = None

        # check if any ne feature was used by the model before computing the pipeline entities and ners features
        if any(key[:2] == 'ne' and value == 1 for key, value in self.used_features.items()):
            pipeline_entities = new_full_pipe(corpus, options={"entity_recognition":True, "string_or_array":True})
            entities = build_sentences_from_tokens(pipeline_entities.entities)
            # compute NERs
            ners = compute_ner(entities)
        else:
            ners = None

        if semantic_relations is None and ners is None:
            return None
        else:
            tmp_semantic_features = []

            for pair in range(len(corpus)//2):
                pair_features = []

                if semantic_relations is not None:
                    if not semantic_relations.empty:
                        if self.used_features['sr_antonyms'] == 1:
                            pair_features.append(semantic_relations['antonyms'][pair])
                        if self.used_features['sr_synonyms'] == 1:
                            pair_features.append(semantic_relations['synonyms'][pair])
                        if self.used_features['sr_hyperonyms'] == 1:
                            pair_features.append(semantic_relations['hyperonyms'][pair])
                        if self.used_features['sr_other'] == 1:
                            pair_features.append(semantic_relations['other'][pair])

                if ners is not None:
                    if not ners.empty:
                        if self.used_features['all_ne'] == 1:
                            pair_features.append(ners['all_ners'][pair])
                        if self.used_features['ne_B-ABSTRACCAO'] == 1:
                            pair_features.append(ners['B-ABSTRACCAO'][pair])
                        if self.used_features['ne_B-ACONTECIMENTO'] == 1:
                            pair_features.append(ners['B-ACONTECIMENTO'][pair])
                        if self.used_features['ne_B-COISA'] == 1:
                            pair_features.append(ners['B-COISA'][pair])
                        if self.used_features['ne_B-LOCAL'] == 1:
                            pair_features.append(ners['B-LOCAL'][pair])
                        if self.used_features['ne_B-OBRA'] == 1:
                            pair_features.append(ners['B-OBRA'][pair])
                        if self.used_features['ne_B-ORGANIZACAO'] == 1:
                            pair_features.append(ners['B-ORGANIZACAO'][pair])
                        if self.used_features['ne_B-OUTRO'] == 1:
                            pair_features.append(ners['B-OUTRO'][pair])
                        if self.used_features['ne_B-PESSOA'] == 1:
                            pair_features.append(ners['B-PESSOA'][pair])
                        if self.used_features['ne_B-TEMPO'] == 1:
                            pair_features.append(ners['B-TEMPO'][pair])
                        if self.used_features['ne_B-VALOR'] == 1:
                            pair_features.append(ners['B-VALOR'][pair])

                pair_features_tuple = tuple(pair_features)
                tmp_semantic_features.append(pair_features_tuple)

            self.semantic_features = np.array(tmp_semantic_features)

            return self.semantic_features

    def extract_distributional_features(self, corpus, tfidf, word2vec_mdl=None, fasttext_mdl=None, ptlkb_mdl=None, glove_mdl=None, numberbatch_mdl=None):
        '''
        Parameters
        ----------


        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return

        if word2vec_mdl:
            word2vec = word2vec_model(word2vec_mdl, corpus, 0, 1, 0)

            word2vec_tfidf = word2vec_model(word2vec_mdl, corpus, 1, 1, 0)
            self.used_features.update({'word2vec': 1, 'word2vec_tfidf': 1})
        else:
            self.used_features.update({'word2vec': 0, 'word2vec_tfidf': 0})

        if fasttext_mdl:
            fasttext = fasttext_model(fasttext_mdl, corpus, 0, 1, 0)

            fasttext_tfidf = fasttext_model(fasttext_mdl, corpus, 1, 1, 0)
            self.used_features.update({'fasttext': 1, 'fasttext_tfidf': 1})
        else:
            self.used_features.update({'fasttext': 0, 'fasttext_tfidf': 0})

        if ptlkb_mdl:
            pipeline_lemmas = new_full_pipe(corpus, options={"lemmatizer":True, "string_or_array":True})
            lemmas = build_sentences_from_tokens(pipeline_lemmas.lemas)

            ptlkb = ptlkb_model(ptlkb_mdl, 0, 1, 0, lemmas)

            ptlkb_tfidf = ptlkb_model(ptlkb_mdl, 1, 1, 0, lemmas)
            self.used_features.update({'ptlkb': 1, 'ptlkb_tfidf': 1})
        else:
            self.used_features.update({'ptlkb': 0, 'ptlkb_tfidf': 0})

        if glove_mdl:
            glove = word_embeddings_model(glove_mdl, corpus, 0, 1, 0)

            glove_tfidf = word_embeddings_model(glove_mdl, corpus, 1, 1, 0)
            self.used_features.update({'glove': 1, 'glove_tfidf': 1})
        else:
            self.used_features.update({'glove': 0, 'glove_tfidf': 0})

        if numberbatch_mdl:
            numberbatch = word_embeddings_model(numberbatch_mdl, corpus, 0, 1, 0)

            numberbatch_tfidf = word_embeddings_model(numberbatch_mdl, corpus, 1, 1, 0)
            self.used_features.update({'numberbatch': 1, 'numberbatch_tfidf': 1})
        else:
            self.used_features.update({'numberbatch': 0, 'numberbatch_tfidf': 0})

        if tfidf:
            # compute tfidf matrix - padding was applied to vectors of different sizes by adding zeros to the smaller vector of the pair
            tfidf_corpus = preprocessing(corpus, 0, 0, 0, 1)
            tfidf_matrix = compute_tfidf_matrix(tfidf_corpus, 0, 0, 1)
            self.used_features.update({'tfidf': 1})
        else:
            self.used_features.update({'tfidf': 0})

        tmp_distributional_features = []

        for pair in range(len(corpus)//2):
            pair_features = []

            if word2vec_mdl:
                pair_features.append(word2vec[pair])
                pair_features.append(word2vec_tfidf[pair])
            else:
                pair_features.extend([0]*2)

            if fasttext_mdl:
                pair_features.append(fasttext[pair])
                pair_features.append(fasttext_tfidf[pair])
            else:
                pair_features.extend([0]*2)

            if ptlkb_mdl:
                pair_features.append(ptlkb[pair])
                pair_features.append(ptlkb_tfidf[pair])
            else:
                pair_features.extend([0]*2)

            if glove_mdl:
                pair_features.append(glove[pair])
                pair_features.append(glove_tfidf[pair])
            else:
                pair_features.extend([0]*2)

            if numberbatch_mdl:
                pair_features.append(numberbatch[pair])
                pair_features.append(numberbatch_tfidf[pair])
            else:
                pair_features.extend([0]*2)

            if tfidf:
                pair_features.append(tfidf_matrix[pair])
            else:
                pair_features.extend([0]*1)

            pair_features_tuple = tuple(pair_features)
            tmp_distributional_features.append(pair_features_tuple)

        self.distributional_features = np.array(tmp_distributional_features)

        return self.distributional_features

    def _extract_distributional_features(self, corpus, word2vec_mdl=None, fasttext_mdl=None, ptlkb_mdl=None, glove_mdl=None, numberbatch_mdl=None):
        '''
        Parameters
        ----------


        '''

        if corpus is None:
            print("Argument corpus is empty, returning None")
            return

        if word2vec_mdl:
            if self.used_features['word2vec'] == 1:
                word2vec = word2vec_model(word2vec_mdl, corpus, 0, 1, 0)
            if self.used_features['word2vec_tfidf'] == 1:
                word2vec_tfidf = word2vec_model(word2vec_mdl, corpus, 1, 1, 0)

        if fasttext_mdl:
            if self.used_features['fasttext'] == 1:
                fasttext = fasttext_model(fasttext_mdl, corpus, 0, 1, 0)
            if self.used_features['fasttext_tfidf'] == 1:
                fasttext_tfidf = fasttext_model(fasttext_mdl, corpus, 1, 1, 0)

        if ptlkb_mdl:
            if self.used_features['ptlkb'] == 1 or self.used_features['ptlkb_tfidf'] == 1:
                pipeline_lemmas = new_full_pipe(corpus, options={"lemmatizer":True, "string_or_array":True})
                lemmas = build_sentences_from_tokens(pipeline_lemmas.lemas)

                if self.used_features['ptlkb'] == 1:
                    ptlkb = ptlkb_model(ptlkb_mdl, 0, 1, 0, lemmas)

                if self.used_features['ptlkb_tfidf'] == 1:
                    ptlkb_tfidf = ptlkb_model(ptlkb_mdl, 1, 1, 0, lemmas)

        if glove_mdl:
            if self.used_features['glove'] == 1:
                glove = word_embeddings_model(glove_mdl, corpus, 0, 1, 0)

            if self.used_features['glove_tfidf'] == 1:
                glove_tfidf = word_embeddings_model(glove_mdl, corpus, 1, 1, 0)

        if numberbatch_mdl:
            if self.used_features['numberbatch'] == 1:
                numberbatch = word_embeddings_model(numberbatch_mdl, corpus, 0, 1, 0)

            if self.used_features['numberbatch_tfidf'] == 1:
                numberbatch_tfidf = word_embeddings_model(numberbatch_mdl, corpus, 1, 1, 0)

        if self.used_features['tfidf'] == 1:
            # compute tfidf matrix - padding was applied to vectors of different sizes by adding zeros to the smaller vector of the pair
            tfidf_corpus = preprocessing(corpus, 0, 0, 0, 1)
            tfidf_matrix = compute_tfidf_matrix(tfidf_corpus, 0, 0, 1)

        tmp_distributional_features = []

        for pair in range(len(corpus)//2):
            pair_features = []

            if word2vec_mdl:
                if self.used_features['word2vec'] == 1:
                    pair_features.append(word2vec[pair])
                if self.used_features['word2vec_tfidf'] == 1:
                    pair_features.append(word2vec_tfidf[pair])

            if fasttext_mdl:
                if self.used_features['fasttext'] == 1:
                    pair_features.append(fasttext[pair])
                if self.used_features['fasttext_tfidf'] == 1:
                    pair_features.append(fasttext_tfidf[pair])

            if ptlkb_mdl:
                if self.used_features['ptlkb'] == 1:
                    pair_features.append(ptlkb[pair])
                if self.used_features['ptlkb_tfidf'] == 1:
                    pair_features.append(ptlkb_tfidf[pair])

            if glove_mdl:
                if self.used_features['glove'] == 1:
                    pair_features.append(glove[pair])
                if self.used_features['glove_tfidf'] == 1:
                    pair_features.append(glove_tfidf[pair])

            if numberbatch_mdl:
                if self.used_features['numberbatch'] == 1:
                    pair_features.append(numberbatch[pair])
                if self.used_features['numberbatch_tfidf'] == 1:
                    pair_features.append(numberbatch_tfidf[pair])

            if self.used_features['tfidf'] == 1:
                pair_features.append(tfidf_matrix[pair])

            pair_features_tuple = tuple(pair_features)
            tmp_distributional_features.append(pair_features_tuple)

        self.distributional_features = np.array(tmp_distributional_features)

        return self.distributional_features

    def extract_multiple_features(self, corpus, to_store=1, extract_lexical=1, extract_syntactic=1, extract_semantic=1, extract_distributional=1, manual_feature_selection=None, word2vec_mdl=None, fasttext_mdl=None, ptlkb_mdl=None, glove_mdl=None, numberbatch_mdl=None):
        '''
        Parameters
        ----------


        '''
        if corpus is None:
            print("Argument corpus is empty, returning None")
            return

        # TODO: It would be awesome if we could just pass a dictionary with the features we want to extract
        if (extract_lexical and extract_syntactic and extract_semantic and extract_distributional) == 0:
            print("All feature groups are set to 0, so no features will be returned")
            return

        if manual_feature_selection is not None:
            self.feature_selection = 1
            self.used_features = self._update_used_features(manual_feature_selection)

        if extract_lexical:
            if self.lexical_features is None or to_store == 0:
                if self.feature_selection == 1:
                    lexical_features = self._extract_lexical_features(corpus)
                else:
                    lexical_features = self.extract_lexical_features(corpus, 1, 1, 1, 1, 1, 1)
            else:
                lexical_features = self.lexical_features
        else:
            lexical_features = None

        if extract_syntactic:
            if self.syntactic_features is None or to_store == 0:
                if self.feature_selection == 1:
                    syntactic_features = self._extract_syntactic_features(corpus)
                else:
                    syntactic_features = self.extract_syntactic_features(corpus, 1, 1)
            else:
                syntactic_features = self.syntactic_features
        else:
            syntactic_features = None

        if extract_semantic:
            if self.semantic_features is None or to_store == 0:
                if self.feature_selection == 1:
                    semantic_features = self._extract_semantic_features(corpus)
                else:
                    semantic_features = self.extract_semantic_features(corpus, 1, 1)
            else:
                semantic_features = self.semantic_features
        else:
            semantic_features = None

        if extract_distributional:
            if self.distributional_features is None or to_store == 0:
                if self.feature_selection == 1:
                    distributional_features = self._extract_distributional_features(corpus, word2vec_mdl, fasttext_mdl, ptlkb_mdl, glove_mdl, numberbatch_mdl)
                else:
                    distributional_features = self.extract_distributional_features(corpus, 1, word2vec_mdl, fasttext_mdl, ptlkb_mdl, glove_mdl, numberbatch_mdl)
            else:
                distributional_features = self.distributional_features
        else:
            distributional_features = None

        # before concatening all groups of features it is necessary to check if they were extracted or not
        all_features = np.concatenate([fg for fg in [lexical_features, syntactic_features, semantic_features, distributional_features] if fg is not None], axis=1)

        # Choose whether the extracted features should be stored in the model or just returned
        if to_store:
            self.all_features = all_features

            return self.all_features
        else:
            return all_features

    # TODO: Should the function return an instance of self, the predicted similarity or both?
    # TODO: Should it return the predicted similarity or only the predict_similarity function should be able to do that
    def run_model(self, regressor, train_features, train_target, use_feature_selection=0, eval_features=None, eval_target=None, test_features=None):
        '''
        Parameters
        ----------


        '''
        if use_feature_selection and self.feature_selection == 0:
            if eval_features is not None and eval_target is not None:
                f_selection_values = feature_selection(train_features, eval_features, train_target, eval_target, regressor, self.used_features)

                # If len(f_selection_values) is different than 3, it means that all feature selection techniques achieve worst performance than unsing all features
                if f_selection_values is not None:
                    selector = f_selection_values[0]
                    selected_train_features = f_selection_values[1]
                    self.used_features = f_selection_values[2]

                    self.model = regressor.fit(selected_train_features, train_target)
                else:
                    self.model = regressor.fit(train_features, train_target)

                self._update_number_features()

                if test_features is not None:
                    selected_features = selector.transform(test_features)
                    predicted_similarity = self.model.predict(selected_features)

                    return predicted_similarity
                else:
                    print("Missing test features. Provide them if you want to test the model.")
            else:
                self._update_number_features()

                print("Missing evaluation features/target necessary to perform feature selection.")
        else:
            self._update_number_features()

            self.model = regressor.fit(train_features, train_target)

            if test_features is not None:
                predicted_similarity = self.model.predict(test_features)

                return predicted_similarity
            else:
                print("Missing test features. Provide them if you want to test the model.")

    def predict_similarity(self, features):
        '''
        Parameters
        ----------


        '''
        # TODO: Does not work without the selector to transform the features in case feature selection was used
        if features is not None:
            predicted_similarity = self.model.predict(features)

            return predicted_similarity
        else:
            print("Features parameter is missing. Provide it in order to make a prediction.")

    # TODO: Verify if it makes sense to add a tests folder to each model in order to store tests performed by it
    def save_model(self):
        '''
        Parameters
        ----------


        '''
        # TODO: Add an option to save the ID of each pair of sentences if needed
        save_model_path = os.path.join(ROOT_PATH, "trained_models", self.model_name)

        if os.path.exists(save_model_path):
            update_model = str(input("Directory {} already exists. Do you want to update the existing model (Y/n)? ".format(save_model_path)))

            if update_model.lower() == "y":
                update_model = 1
            else:
                update_model = 0

        # TODO: When saving files, check if the content to be written is the same that already exists in the file and if so don't write it again. It could be done using an hash.
        if not os.path.exists(save_model_path) or update_model:

            if not os.path.exists(save_model_path):
                os.mkdir(save_model_path)

            if self.model is not None:
                sts_model_path = os.path.join(save_model_path, self.model_name)

                dump(self.model, sts_model_path)

            if self.lexical_features is not None:
                lexical_features_path = os.path.join(save_model_path, 'lexical_features.csv')

                np.savetxt(lexical_features_path, self.lexical_features, delimiter=",")

            if self.syntactic_features is not None:
                syntactic_features_path = os.path.join(save_model_path, 'syntactic_features.csv')

                np.savetxt(syntactic_features_path, self.syntactic_features, delimiter=",")

            if self.semantic_features is not None:
                semantic_features_path = os.path.join(save_model_path, 'semantic_features.csv')

                np.savetxt(semantic_features_path, self.semantic_features, delimiter=",")

            if self.distributional_features is not None:
                distributional_features_path = os.path.join(save_model_path, 'distributional_features.csv')

                np.savetxt(distributional_features_path, self.distributional_features, delimiter=",")

            if self.all_features is not None:
                all_features_path = os.path.join(save_model_path, 'all_features.csv')

                np.savetxt(all_features_path, self.all_features, delimiter=",")

            if self.used_features:
                used_features_path = os.path.join(save_model_path, 'used_features.txt')

                with open(used_features_path, 'w') as fp:
                    fp.write('{}\n'.format(self.model_name))
                    fp.write('{}\n'.format(self.number_features))

                    for key, value in self.used_features.items():
                        fp.write('{}: {}\n'.format(key, value))

                fp.close()

    def load_model(self, model_name):
        '''
        Parameters
        ----------


        '''
        dir_load_model_path = os.path.join(ROOT_PATH, 'trained_models', model_name)

        while not os.path.exists(dir_load_model_path):
            model_name = str(input("Directory {} does not exist. Insert a valid model name:".format(dir_load_model_path)))

            dir_load_model_path = os.path.join(ROOT_PATH, 'trained_models', model_name)

        self.model_name = model_name

        load_model_path = os.path.join(dir_load_model_path, model_name)
        self.model = load(load_model_path)

        used_features_path = os.path.join(dir_load_model_path, 'used_features.txt')
        if os.path.exists(used_features_path):
            self.feature_selection = 1

            with open(used_features_path) as ufp:
                for i, line in enumerate(ufp):
                    if i == 1:
                        self.number_features = int(line)
                    if i > 1:
                        split_line = line.split(': ')
                        self.used_features[split_line[0]] = int(split_line[1][:-1])

        lexical_features_path = os.path.join(dir_load_model_path, 'lexical_features.csv')
        if os.path.exists(lexical_features_path):
            self.lexical_features = np.loadtxt(lexical_features_path, delimiter=",")

        syntactic_features_path = os.path.join(dir_load_model_path, 'syntactic_features.csv')
        if os.path.exists(syntactic_features_path):
            self.syntactic_features = np.loadtxt(syntactic_features_path, delimiter=",")

        semantic_features_path = os.path.join(dir_load_model_path, 'semantic_features.csv')
        if os.path.exists(semantic_features_path):
            self.semantic_features = np.loadtxt(semantic_features_path, delimiter=",")

        distributional_features_path = os.path.join(dir_load_model_path, 'distributional_features.csv')
        if os.path.exists(distributional_features_path):
            self.distributional_features = np.loadtxt(distributional_features_path, delimiter=",")

        all_features_path = os.path.join(dir_load_model_path, 'all_features.csv')
        if os.path.exists(all_features_path):
            self.all_features = np.loadtxt(all_features_path, delimiter=",")

    def _update_number_features(self):
        '''
        Parameters
        ----------


        '''
        tmp_features = 0

        for value in self.used_features.values():
            if value == 1:
                tmp_features += 1

        self.number_features = tmp_features

    @staticmethod
    def _update_used_features(manual_features_selected):
        used_features = {'wn_jaccard_1': 1, 'wn_jaccard_2': 1, 'wn_jaccard_3': 1, 'wn_dice_1': 1, 'wn_dice_2': 1, 'wn_dice_3': 1, 'wn_overlap_1': 1, 'wn_overlap_2': 1, 'wn_overlap_3': 1, 'cn_jaccard_2': 1, 'cn_jaccard_3': 1, 'cn_jaccard_4': 1, 'cn_dice_2': 1, 'cn_dice_3': 1, 'cn_dice_4': 1, 'cn_overlap_2': 1, 'cn_overlap_3': 1, 'cn_overlap_4': 1, 'pos_adj': 1, 'pos_adv': 1, 'pos_art': 1, 'pos_conj-c': 1, 'pos_conj-s': 1, 'pos_intj': 1, 'pos_n': 1, 'pos_n-adj': 1, 'pos_num': 1, 'pos_pron-det': 1, 'pos_pron-indp': 1, 'pos_pron-pers': 1, 'pos_prop': 1, 'pos_prp': 1, 'pos_punc': 1, 'pos_v-fin': 1, 'pos_v-ger': 1, 'pos_v-inf': 1, 'pos_v-pcp': 1, 'dependency_parsing': 1, 'sr_antonyms': 1, 'sr_synonyms': 1, 'sr_hyperonyms': 1, 'sr_other': 1, 'all_ne': 1, 'ne_B-ABSTRACCAO': 1, 'ne_B-ACONTECIMENTO': 1, 'ne_B-COISA': 1, 'ne_B-LOCAL': 1, 'ne_B-OBRA': 1, 'ne_B-ORGANIZACAO': 1, 'ne_B-OUTRO': 1, 'ne_B-PESSOA': 1, 'ne_B-TEMPO': 1, 'ne_B-VALOR': 1, 'word2vec': 1, 'word2vec_tfidf': 1, 'fasttext': 1, 'fasttext_tfidf': 1, 'ptlkb': 1, 'ptlkb_tfidf': 1, 'glove': 1, 'glove_tfidf': 1, 'numberbatch': 1, 'numberbatch_tfidf': 1, 'tfidf': 1}

        for key, value in manual_features_selected.items():
            if key in used_features.keys():
                used_features[key] = value

        return used_features



from ASAPPpy.scripts.tools import build_sentences_from_tokens

from NLPyPort.FullPipeline import new_full_pipe

class Sentence():
    def __init__(self, text, tags=None, lemmas=None, entities=None):
        self.text = text
        self.tags = tags
        self.lemmas = lemmas
        self.entities = entities

    def compute_tags(self):
        pipeline_tags = new_full_pipe([self.text], options={"pos_tagger":True, "string_or_array":True})
        tags = build_sentences_from_tokens(pipeline_tags.pos_tags)

        # The pipeline output is a list, so we access the first element to obtain the sentence.
        self.tags = tags[0]

    def compute_lemmas(self):
        pipeline_lemmas = new_full_pipe([self.text], options={"lemmatizer":True, "string_or_array":True})
        lemmas = build_sentences_from_tokens(pipeline_lemmas.lemas)

        # The pipeline output is a list, so we access the first element to obtain the sentence.
        self.lemmas = lemmas[0]

    def compute_entities(self):
        pipeline_entities = new_full_pipe([self.text], options={"entity_recognition":True, "string_or_array":True})
        entities = build_sentences_from_tokens(pipeline_entities.entities)

        # The pipeline output is a list, so we access the first element to obtain the sentence.
        self.entities = entities[0]

    def compute_all(self):
        pipeline_output = new_full_pipe([self.text], options={"string_or_array":True})

        tags = build_sentences_from_tokens(pipeline_output.pos_tags)
        lemmas = build_sentences_from_tokens(pipeline_output.lemas)
        entities = build_sentences_from_tokens(pipeline_output.entities)

        # The pipeline output is a list, so we access the first element to obtain the sentence.
        self.tags = tags[0]
        self.lemmas = lemmas[0]
        self.entities = entities[0]
        


import keras as k
import pandas as pd
import cufflinks as cf
from sklearn.model_selection import train_test_split
from keras import Sequential
from keras.layers import  Embedding
from keras.layers import Dense,SpatialDropout1D
from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences
from keras.layers.recurrent import LSTM
import numpy as np 
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences
from keras.models import Sequential
from keras.layers import Dense, Embedding, LSTM, SpatialDropout1D
from sklearn.model_selection import train_test_split
from keras.utils.np_utils import to_categorical
from keras.callbacks import EarlyStopping
from keras.layers import Dropout
from keras.models import Model, load_model
from sklearn.utils import resample
import tensorflow as tf
from keras.optimizers import Adam, RMSprop
from imblearn.over_sampling import SMOTE
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.naive_bayes import MultinomialNB
from sklearn.metrics import classification_report
from sklearn.naive_bayes import GaussianNB
import pickle
from sklearn import svm
from sklearn.feature_extraction.text import TfidfVectorizer
#naive bayes, svm, random forest



#https://github.com/susanli2016/NLP-with-Python/blob/master/Multi-Class%20Text%20Classification%20LSTM%20Consumer%20complaints.ipynb

#1---Empresa na Hora
#2---Marca na hora e marca na hora online
#3---Formas jurídicas
#4---Cartão de Empresa/Cartão de Pessoa Coletiva
#5---Criação da Empresa Online
#6---Certificados de Admissibilidade
#7---Inscrições Online
#8---Certidões Online
#9---Gestão da Empresa Online
#10---RJACSR
#11---Alojamento Local

def escala(X,y):
	from imblearn.over_sampling import RandomOverSampler
	sampler = RandomOverSampler(sampling_strategy='not majority',random_state=0)
	X_train, Y_train = sampler.fit_sample(X, y)
	return X_train,Y_train

def treina(model_name):

	df = pd.read_csv("divididobinario.txt",sep='§',header=0)
	df.info()

	max_len = 0
	for value in df.Perguntas:
		if(len(value)>max_len):
			#print(value)
			max_len = len(value)
	max_words = 0
	for value in df.Perguntas:
		word_count = len(value.split(" "))
		if(word_count>max_words):
			#print(word_count)
			max_words = word_count
	#print("---------")
	#print(max_words)
	#print(df.Class.value_counts().sort_values())
	g = df.groupby('Class')
	g= pd.DataFrame(g.apply(lambda x: x.sample(g.size().min()).reset_index(drop=True)))
	#print(g)

	#df =  pd.DataFrame(g.apply(lambda x: x.sample(g.size().min()).reset_index(drop=True)))
	#print("###")
	#print(df.shape)
	#print(df.head(10))
	#df = g



	#X_conveted = pd.get_dummies(df["Perguntas"])
	#Divisão em treino e teste
	X_train, X_test, Y_train, Y_test = train_test_split(df["Perguntas"],df["Class"], test_size = 0.3, random_state = 42)
	#print(X_train.shape)
	#print(Y_train.shape)
	#print(X_test.shape)
	#print(Y_test.shape)



	vect = TfidfVectorizer().fit(X_train)

	with open("vect_bin", 'wb') as fid:
		pickle.dump(vect, fid)	

	X_train_vectorized = vect.transform(X_train)

	X_train_vectorized, Y_train = escala(X_train_vectorized, Y_train)

	clf = svm.SVC(gamma="scale",kernel='linear', degree=16)
	clf.fit(X_train_vectorized,Y_train)

	with open(model_name, 'wb') as fid:
		pickle.dump(clf, fid)	

	preds = clf.predict(vect.transform(X_test))
	score = classification_report(Y_test, preds)
	print(score)

	return vect


def corre_para_frase_1(modelo,frase):
	with open(modelo, 'rb') as fid:
		clfrNB = pickle.load(fid)

	with open("classifiers/vectors/vect_bin", 'rb') as fid:
		v = pickle.load(fid)
	sentences = [frase]
	print("Entrada recebida.")
	transformada = v.transform(sentences)
	preds = clfrNB.predict(transformada)


	############################################
	#Parte para alimentar o outro classificador
	############################################
	in_domain = []
	for index,value in enumerate(preds):
		print(value)
		if(value==1):
			#frase = str(sentences[index])+"§"+str(classe_original[index])
			in_domain.append(frase)
	return in_domain

def corre_para_frase(frase):
	with open("classifiers/trained_models/svm_binaria_v3.pickle", 'rb') as fid:
		clfrNB = pickle.load(fid)

	with open("classifiers/vectors/vect_bin", 'rb') as fid:
		v = pickle.load(fid)
	sentences = [frase]
	print("Entrada recebida.")
	transformada = v.transform(sentences)
	preds = clfrNB.predict(transformada)

	'''
	############################################
	#Parte para alimentar o outro classificador
	############################################
	in_domain = []
	for index,value in enumerate(preds):
		print(value)
		if(value==1):
			#frase = str(sentences[index])+"§"+str(classe_original[index])
			in_domain.append(frase)
	return in_domain
	'''
	return preds[0]

def corre_para_testes(modelo,ficheiro):
	print("##############")
	nome = ficheiro.replace("out_","")
	nome = nome.replace(".txt","")
	print(nome.upper())
	print("##############")
	with open(modelo, 'rb') as fid:
		clfrNB = pickle.load(fid)

	with open("vect_bin", 'rb') as fid:
		v = pickle.load(fid)
	sentences = []
	true_results = []
	classe_original = []
	with open(ficheiro,"r") as f:
		for line in f:
			line = line.replace("\n","")
			line = line.split("§")
			sentences.append(line[0])
			classe_original.append(int(line[1]))
			if(int(line[1])<15):
				true_results.append(1)
			else:
				true_results.append(15)
	print("Entrada recebida.")
	transformada = v.transform(sentences)
	preds = clfrNB.predict(transformada)

	score = classification_report(true_results, preds,labels=[1,15])
	print(score)


	############################################
	#Parte para alimentar o outro classificador
	############################################
	in_domain = []
	for index,value in enumerate(preds):
		if(value==1):
			frase = str(sentences[index])+"§"+str(classe_original[index])
			in_domain.append(frase)
	return in_domain

def corre(modelo,v):
	labels = ["In the domain","In the domain","Formas jurídicas","Cartão de Empresa/Cartão de Pessoa Coletiva","Criação da Empresa Online","Certificados de Admissibilidade","Inscrições Online","Certidões Online","Gestão da Empresa Online","RJACSR","Alojamento Local","Out of Domain","Out of Domain","Out of Domain","Out of Domain","Out of Domain","Out of Domain"]
	
	with open(modelo, 'rb') as fid:
		clfrNB = pickle.load(fid)

	with open("vect_bin", 'rb') as fid:
		v = pickle.load(fid)
	a = 0

	while (a==0):
		print("Entrada.")
		entrada = input()
		entrada = [entrada]
		#entrada = pd.DataFrame([entrada])
		print("Entrada recebida.")
		transformada = v.transform(entrada)
		preds = clfrNB.predict(transformada)
		print(preds)
		if(preds[0]==1):
			print("ID")
		else:
			print("OOD")



if __name__ == '__main__':
	#_lr_0.03
	#modelo = "lstm_com_balanceamento_varias_camadas_500_lr_0.03.h5"
	#modelo = "lstm_com_balanceamento_varias_camadas_200_lr_0.03.h5"
	#vect = treina("modelos/svm_binaria.pickle")
	#corre("modelos/svm_binaria.pickle","")
	#corre_para_testes("modelos/svm_binaria.pickle","inputs/out_vg1_legendas_original.txt")
	print(corre_para_frase("modelos/svm_binaria.pickle","frase de teste"))
	print(corre_para_frase("modelos/svm_binaria.pickle","akhjsd jkasndkjasnjdknaskj nda"))







'''
Module used for feature extraction of a corpus.
'''

import time
import os
import collections
import argparse
import spacy
import numpy as np
from gensim.models import KeyedVectors
from xml.etree import cElementTree as ET
import pandas as pd
from sklearn.linear_model import LinearRegression
from sklearn.gaussian_process import GaussianProcessRegressor
from sklearn.svm import SVR
from sklearn.tree import DecisionTreeRegressor
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.ensemble import RandomForestRegressor
from sklearn.ensemble import BaggingRegressor
from sklearn.ensemble import AdaBoostRegressor
from sklearn.feature_selection import SelectKBest
from sklearn.feature_selection import f_regression
from sklearn.feature_selection import SelectPercentile
from sklearn.feature_selection import SelectFromModel
from sklearn.feature_selection import RFE
from sklearn.feature_selection import RFECV
from sklearn.ensemble import VotingRegressor
from sklearn.model_selection import GridSearchCV
from joblib import dump, load

from NLPyPort.FullPipeline import new_full_pipe

from ASAPPpy import ROOT_PATH

from .scripts.xml_reader import read_xml
from .scripts.xml_reader import read_xml_no_attributes

# import models.word2vec.word2vec as w2c
# import models.fastText.fasttext as ftt
# import models.ontoPT.ptlkb as ptlkb
# import load_embeddings as ld_emb
# import tools as tl

from .models.word2vec.word2vec import word2vec_model
from .models.fastText.fasttext import fasttext_model
from .models.ontoPT.ptlkb import ptlkb_model
from .load_embeddings import word_embeddings_model
from .scripts.tools import preprocessing, compute_tfidf_matrix, read_corpus, write_features_to_csv

from .feature_engineering.lexical_features import create_word_ngrams, create_multiple_word_ngrams, create_character_ngrams, create_multiple_character_ngrams, compute_jaccard, compute_dice, compute_overlap, NG
from .feature_engineering.syntactic_features import compute_pos, dependency_parsing
from .feature_engineering.semantic_features import compute_ner, compute_semantic_relations

def build_sentences_from_tokens(tokens):
	""" Function used to rebuild the sentences from the tokens returned by the pipeline """

	sentences = []
	tmp_sentence = []

	for elem in tokens:
		if elem == "EOS":
			tmp_sentence = ' '.join(tmp_sentence)
			sentences.append(tmp_sentence)
			tmp_sentence = []
		else:
			tmp_sentence.append(elem)

	return sentences

def extract_features(run_pipeline, corpus, preprocessed_corpus, word2vec_mdl=None, fasttext_mdl=None, ptlkb64_mdl=None, glove300_mdl=None, numberbatch_mdl=None, f_selection=None):
	""" Function used to extract the features """

	# run NLPyPort pipeline before extracting the features
	if run_pipeline == 1:
		# if system_mode == 0:
		# 	corpus_path = os.path.join('datasets', 'FAQ_todas_variantes_texto_clean.txt')
		# elif system_mode == 1:
		# 	corpus_path = os.path.join('NLPyPort', 'SampleInput', 'train_corpus_ptpt_ptbr.txt')
		# elif system_mode == 2:
		# 	corpus_path = os.path.join('NLPyPort', 'SampleInput', 'train_test_corpus_ptpt_ptbr.txt')
		# elif system_mode == 3:
		# 	corpus_path = "tmp_file.txt"
		# elif system_mode == 4:
		# 	corpus_path = os.path.join('NLPyPort', 'SampleInput', 'complete_assin_training_corpus.txt')
		# elif system_mode == 5:
		# 	corpus_path = os.path.join('NLPyPort', 'SampleInput', 'complete_assin_training_assin1_testing_corpus.txt')

		start_time = time.time()
		print("Started running the pipeline")

		pipeline_output = new_full_pipe(corpus, options={"string_or_array":True})

		# there's still a bug in the pipeline that makes it output two EOS tokens at the end of each run, reason why we read the output to the penultimate token.
		tags = build_sentences_from_tokens(pipeline_output.pos_tags)
		lemmas = build_sentences_from_tokens(pipeline_output.lemas)
		entities = build_sentences_from_tokens(pipeline_output.entities)

		print("Finished running the pipeline successfully")
		print("--- %s seconds ---" %(time.time() - start_time))
		print('\a')

	features = []

	# create word ngrams of different sizes
	if (f_selection is None) or (1 in f_selection[0:9:3]):
		word_ngrams_1 = create_word_ngrams(preprocessed_corpus, 1)

	if (f_selection is None) or (1 in f_selection[1:9:3]):
		word_ngrams_2 = create_word_ngrams(preprocessed_corpus, 2)

	if (f_selection is None) or (1 in f_selection[2:9:3]):
		word_ngrams_3 = create_word_ngrams(preprocessed_corpus, 3)

	# compute distance coefficients for these ngrams
	if (f_selection is None) or f_selection[0]:
		wn_jaccard_1 = compute_jaccard(word_ngrams_1)
	if (f_selection is None) or f_selection[1]:
		wn_jaccard_2 = compute_jaccard(word_ngrams_2)
	if (f_selection is None) or f_selection[2]:
		wn_jaccard_3 = compute_jaccard(word_ngrams_3)

	if (f_selection is None) or f_selection[3]:
		wn_dice_1 = compute_dice(word_ngrams_1)
	if (f_selection is None) or f_selection[4]:
		wn_dice_2 = compute_dice(word_ngrams_2)
	if (f_selection is None) or f_selection[5]:
		wn_dice_3 = compute_dice(word_ngrams_3)

	if (f_selection is None) or f_selection[6]:
		wn_overlap_1 = compute_overlap(word_ngrams_1)
	if (f_selection is None) or f_selection[7]:
		wn_overlap_2 = compute_overlap(word_ngrams_2)
	if (f_selection is None) or f_selection[8]:
		wn_overlap_3 = compute_overlap(word_ngrams_3)

	# create character ngrams of different sizes
	if (f_selection is None) or (1 in f_selection[9:18:3]):
		character_ngrams_2 = create_character_ngrams(preprocessed_corpus, 2)
	if (f_selection is None) or (1 in f_selection[10:18:3]):
		character_ngrams_3 = create_character_ngrams(preprocessed_corpus, 3)
	if (f_selection is None) or (1 in f_selection[11:18:3]):
		character_ngrams_4 = create_character_ngrams(preprocessed_corpus, 4)

	# compute distance coefficients for these ngrams
	if (f_selection is None) or f_selection[9]:
		cn_jaccard_2 = compute_jaccard(character_ngrams_2)
	if (f_selection is None) or f_selection[10]:
		cn_jaccard_3 = compute_jaccard(character_ngrams_3)
	if (f_selection is None) or f_selection[11]:
		cn_jaccard_4 = compute_jaccard(character_ngrams_4)

	if (f_selection is None) or f_selection[12]:
		cn_dice_2 = compute_dice(character_ngrams_2)
	if (f_selection is None) or f_selection[13]:
		cn_dice_3 = compute_dice(character_ngrams_3)
	if (f_selection is None) or f_selection[14]:
		cn_dice_4 = compute_dice(character_ngrams_4)

	if (f_selection is None) or f_selection[15]:
		cn_overlap_2 = compute_overlap(character_ngrams_2)
	if (f_selection is None) or f_selection[16]:
		cn_overlap_3 = compute_overlap(character_ngrams_3)
	if (f_selection is None) or f_selection[17]:
		cn_overlap_4 = compute_overlap(character_ngrams_4)

	if word2vec_mdl:
		if (f_selection is None) or f_selection[18]:
			word2vec = word2vec_model(word2vec_mdl, corpus, 0, 1, 0)
		if (f_selection is None) or f_selection[19]:
			word2vec_tfidf = word2vec_model(word2vec_mdl, corpus, 1, 1, 0)

	if fasttext_mdl:
		if (f_selection is None) or f_selection[20]:
			fasttext = fasttext_model(fasttext_mdl, corpus, 0, 1, 0)
		if (f_selection is None) or f_selection[21]:
			fasttext_tfidf = fasttext_model(fasttext_mdl, corpus, 1, 1, 0)

	if ptlkb64_mdl:
		if (f_selection is None) or f_selection[22]:
			# if run_pipeline == 0:
			# 	ptlkb_64 = ptlkb.word_embeddings_model(run_pipeline, system_mode, ptlkb64_mdl, 0, 1, 0)
			# else:
			ptlkb_64 = ptlkb_model(ptlkb64_mdl, 0, 1, 0, lemmas)
		if (f_selection is None) or f_selection[23]:
			# if run_pipeline == 0:
			# 	ptlkb_64_tfidf = ptlkb.word_embeddings_model(run_pipeline, system_mode, ptlkb64_mdl, 1, 1, 0)
			# else:
			ptlkb_64_tfidf = ptlkb_model(ptlkb64_mdl, 1, 1, 0, lemmas)

	if glove300_mdl:
		if (f_selection is None) or f_selection[24]:
			glove_300 = word_embeddings_model(glove300_mdl, corpus, 0, 1, 0)
		if (f_selection is None) or f_selection[25]:
			glove_300_tfidf = word_embeddings_model(glove300_mdl, corpus, 1, 1, 0)

	# compute tfidf matrix - padding was applied to vectors of different sizes by adding zeros to the smaller vector of the pair
	if (f_selection is None) or f_selection[26]:
		tfidf_corpus = preprocessing(corpus, 0, 0, 0, 1)
		tfidf_matrix = compute_tfidf_matrix(tfidf_corpus, 0, 0, 1)

	# compute semantic relations coefficients
	if (f_selection is None) or (1 in f_selection[27:31]):
		# relations_file_path = os.path.join('semantic_relations', 'triplos_10recs', 'triplos_todos_10recs_n.txt')
		# if run_pipeline == 0:
		# 	semantic_relations = compute_semantic_relations(run_pipeline, system_mode, relations_file_path, 3)
		# else:
		semantic_relations = compute_semantic_relations(lemmas)

	# compute POS tags
	if (f_selection is None) or (1 in f_selection[31:50]):
		# if run_pipeline == 0:
		# 	pos_tags = compute_pos(run_pipeline, system_mode)
		# else:
		pos_tags = compute_pos(tags)

	# compute NERs
	if (f_selection is None) or (1 in f_selection[50:61]):
		# if run_pipeline == 0:
		# 	ners = compute_ner(run_pipeline, system_mode)
		# else:
		ners = compute_ner(entities)

	# compute Syntactic Dependency parsing
	if (f_selection is None) or f_selection[61]:
		dependencies = dependency_parsing(corpus)

	# create multiple word ngrams of different sizes
	if (f_selection is None) or f_selection[62]:
		word_ngrams_1_2 = create_multiple_word_ngrams(preprocessed_corpus, 1, 2)

	# compute the cosine similarity between the multiple word ngrams converted sentences
	if (f_selection is None) or f_selection[62]:
		wn_cosine_1_2 = NG(word_ngrams_1_2)

	# create multiple character ngrams of different sizes
	if (f_selection is None) or f_selection[63]:
		character_ngrams_1_2_3 = create_multiple_character_ngrams(preprocessed_corpus, 1, 2, 3)

	# compute the cosine similarity between the multiple character ngrams converted sentences
	if (f_selection is None) or f_selection[63]:
		cn_cosine_1_2_3 = NG(character_ngrams_1_2_3)

	if numberbatch_mdl:
		if (f_selection is None) or f_selection[64]:
			numberbatch = word_embeddings_model(numberbatch_mdl, corpus, 0, 1, 0)
		if (f_selection is None) or f_selection[65]:
			numberbatch_tfidf = word_embeddings_model(numberbatch_mdl, corpus, 1, 1, 0)

	for pair in range(len(preprocessed_corpus)):
		if f_selection is not None:
			features_pair = []

			if f_selection[0]:
				features_pair.append(wn_jaccard_1['jaccard'][pair])
			if f_selection[1]:
				features_pair.append(wn_jaccard_2['jaccard'][pair])
			if f_selection[2]:
				features_pair.append(wn_jaccard_3['jaccard'][pair])
			if f_selection[3]:
				features_pair.append(wn_dice_1['dice'][pair])
			if f_selection[4]:
				features_pair.append(wn_dice_2['dice'][pair])
			if f_selection[5]:
				features_pair.append(wn_dice_3['dice'][pair])
			if f_selection[6]:
				features_pair.append(wn_overlap_1['overlap'][pair])
			if f_selection[7]:
				features_pair.append(wn_overlap_2['overlap'][pair])
			if f_selection[8]:
				features_pair.append(wn_overlap_3['overlap'][pair])
			if f_selection[9]:
				features_pair.append(cn_jaccard_2['jaccard'][pair])
			if f_selection[10]:
				features_pair.append(cn_jaccard_3['jaccard'][pair])
			if f_selection[11]:
				features_pair.append(cn_jaccard_4['jaccard'][pair])
			if f_selection[12]:
				features_pair.append(cn_dice_2['dice'][pair])
			if f_selection[13]:
				features_pair.append(cn_dice_3['dice'][pair])
			if f_selection[14]:
				features_pair.append(cn_dice_4['dice'][pair])
			if f_selection[15]:
				features_pair.append(cn_overlap_2['overlap'][pair])
			if f_selection[16]:
				features_pair.append(cn_overlap_3['overlap'][pair])
			if f_selection[17]:
				features_pair.append(cn_overlap_4['overlap'][pair])
			if f_selection[18]:
				features_pair.append(word2vec[pair])
			if f_selection[19]:
				features_pair.append(word2vec_tfidf[pair])
			if f_selection[20]:
				features_pair.append(fasttext[pair])
			if f_selection[21]:
				features_pair.append(fasttext_tfidf[pair])
			if f_selection[22]:
				features_pair.append(ptlkb_64[pair])
			if f_selection[23]:
				features_pair.append(ptlkb_64_tfidf[pair])
			if f_selection[24]:
				features_pair.append(glove_300[pair])
			if f_selection[25]:
				features_pair.append(glove_300_tfidf[pair])
			if f_selection[26]:
				features_pair.append(tfidf_matrix[pair])
			if f_selection[27]:
				features_pair.append(semantic_relations['antonyms'][pair])
			if f_selection[28]:
				features_pair.append(semantic_relations['synonyms'][pair])
			if f_selection[29]:
				features_pair.append(semantic_relations['hyperonyms'][pair])
			if f_selection[30]:
				features_pair.append(semantic_relations['other'][pair])
			if f_selection[31] and ('n' in pos_tags.columns):
				features_pair.append(pos_tags['n'][pair])
			if f_selection[32] and ('prop' in pos_tags.columns):
				features_pair.append(pos_tags['prop'][pair])
			if f_selection[33] and ('adj' in pos_tags.columns):
				features_pair.append(pos_tags['adj'][pair])
			if f_selection[34] and ('n-adj' in pos_tags.columns):
				features_pair.append(pos_tags['n-adj'][pair])
			if f_selection[35] and ('v-fin' in pos_tags.columns):
				features_pair.append(pos_tags['v-fin'][pair])
			if f_selection[36] and ('v-inf' in pos_tags.columns):
				features_pair.append(pos_tags['v-inf'][pair])
			if f_selection[37] and ('v-pcp' in pos_tags.columns):
				features_pair.append(pos_tags['v-pcp'][pair])
			if f_selection[38] and ('v-ger' in pos_tags.columns):
				features_pair.append(pos_tags['v-ger'][pair])
			if f_selection[39] and ('art' in pos_tags.columns):
				features_pair.append(pos_tags['art'][pair])
			if f_selection[40] and ('pron-pers' in pos_tags.columns):
				features_pair.append(pos_tags['pron-pers'][pair])
			if f_selection[41] and ('pron-det' in pos_tags.columns):
				features_pair.append(pos_tags['pron-det'][pair])
			if f_selection[42] and ('pron-indp' in pos_tags.columns):
				features_pair.append(pos_tags['pron-indp'][pair])
			if f_selection[43] and ('adv' in pos_tags.columns):
				features_pair.append(pos_tags['adv'][pair])
			if f_selection[44] and ('num' in pos_tags.columns):
				features_pair.append(pos_tags['num'][pair])
			if f_selection[45] and ('prp' in pos_tags.columns):
				features_pair.append(pos_tags['prp'][pair])
			if f_selection[46] and ('intj' in pos_tags.columns):
				features_pair.append(pos_tags['intj'][pair])
			if f_selection[47] and ('conj-s' in pos_tags.columns):
				features_pair.append(pos_tags['conj-s'][pair])
			if f_selection[48] and ('conj-c' in pos_tags.columns):
				features_pair.append(pos_tags['conj-c'][pair])
			if f_selection[49] and ('punc' in pos_tags.columns):
				features_pair.append(pos_tags['punc'][pair])
			if f_selection[50] and ('all_ners' in ners.columns):
				features_pair.append(ners['all_ners'][pair])
			if f_selection[51] and ('B-ABSTRACCAO' in ners.columns):
				features_pair.append(ners['B-ABSTRACCAO'][pair])
			if f_selection[52] and ('B-ACONTECIMENTO' in ners.columns):
				features_pair.append(ners['B-ACONTECIMENTO'][pair])
			if f_selection[53] and ('B-COISA' in ners.columns):
				features_pair.append(ners['B-COISA'][pair])
			if f_selection[54] and ('B-LOCAL' in ners.columns):
				features_pair.append(ners['B-LOCAL'][pair])
			if f_selection[55] and ('B-OBRA' in ners.columns):
				features_pair.append(ners['B-OBRA'][pair])
			if f_selection[56] and ('B-ORGANIZACAO' in ners.columns):
				features_pair.append(ners['B-ORGANIZACAO'][pair])
			if f_selection[57] and ('B-OUTRO' in ners.columns):
				features_pair.append(ners['B-OUTRO'][pair])
			if f_selection[58] and ('B-PESSOA' in ners.columns):
				features_pair.append(ners['B-PESSOA'][pair])
			if f_selection[59] and ('B-TEMPO' in ners.columns):
				features_pair.append(ners['B-TEMPO'][pair])
			if f_selection[60] and ('B-VALOR' in ners.columns):
				features_pair.append(ners['B-VALOR'][pair])
			if f_selection[61]:
				features_pair.append(dependencies['dependency_parsing_jc'][pair])
			if f_selection[62]:
				features_pair.append(wn_cosine_1_2['NG'][pair])
			if f_selection[63]:
				features_pair.append(cn_cosine_1_2_3['NG'][pair])
			if f_selection[64]:
				features_pair.append(numberbatch[pair])
			if f_selection[65]:
				features_pair.append(numberbatch_tfidf[pair])

			tuple_features_pair = tuple(features_pair)
			features.append(tuple_features_pair)

		else:
			flag = 1
			features_pair = []
			used_features = [False] * 66

			if flag == 1:
				used_features[0] = True
				features_pair.append(wn_jaccard_1['jaccard'][pair])
			if flag == 1:
				used_features[1] = True
				features_pair.append(wn_jaccard_2['jaccard'][pair])
			if flag == 1:
				used_features[2] = True
				features_pair.append(wn_jaccard_3['jaccard'][pair])
			if flag == 1:
				used_features[3] = True
				features_pair.append(wn_dice_1['dice'][pair])
			if flag == 1:
				used_features[4] = True
				features_pair.append(wn_dice_2['dice'][pair])
			if flag == 1:
				used_features[5] = True
				features_pair.append(wn_dice_3['dice'][pair])
			if flag == 1:
				used_features[6] = True
				features_pair.append(wn_overlap_1['overlap'][pair])
			if flag == 1:
				used_features[7] = True
				features_pair.append(wn_overlap_2['overlap'][pair])
			if flag == 1:
				used_features[8] = True
				features_pair.append(wn_overlap_3['overlap'][pair])
			if flag == 1:
				used_features[9] = True
				features_pair.append(cn_jaccard_2['jaccard'][pair])
			if flag == 1:
				used_features[10] = True
				features_pair.append(cn_jaccard_3['jaccard'][pair])
			if flag == 1:
				used_features[11] = True
				features_pair.append(cn_jaccard_4['jaccard'][pair])
			if flag == 1:
				used_features[12] = True
				features_pair.append(cn_dice_2['dice'][pair])
			if flag == 1:
				used_features[13] = True
				features_pair.append(cn_dice_3['dice'][pair])
			if flag == 1:
				used_features[14] = True
				features_pair.append(cn_dice_4['dice'][pair])
			if flag == 1:
				used_features[15] = True
				features_pair.append(cn_overlap_2['overlap'][pair])
			if flag == 1:
				used_features[16] = True
				features_pair.append(cn_overlap_3['overlap'][pair])
			if flag == 1:
				used_features[17] = True
				features_pair.append(cn_overlap_4['overlap'][pair])
			if flag == 0:
				used_features[18] = True
				features_pair.append(word2vec[pair])
			if flag == 0:
				used_features[19] = True
				features_pair.append(word2vec_tfidf[pair])
			if flag == 1:
				used_features[20] = True
				features_pair.append(fasttext[pair])
			if flag == 1:
				used_features[21] = True
				features_pair.append(fasttext_tfidf[pair])
			if flag == 1:
				used_features[22] = True
				features_pair.append(ptlkb_64[pair])
			if flag == 1:
				used_features[23] = True
				features_pair.append(ptlkb_64_tfidf[pair])
			if flag == 1:
				used_features[24] = True
				features_pair.append(glove_300[pair])
			if flag == 1:
				used_features[25] = True
				features_pair.append(glove_300_tfidf[pair])
			if flag == 0:
				used_features[26] = True
				features_pair.append(tfidf_matrix[pair])
			if flag == 0:
				used_features[27] = True
				features_pair.append(semantic_relations['antonyms'][pair])
			if flag == 1:
				used_features[28] = True
				features_pair.append(semantic_relations['synonyms'][pair])
			if flag == 1:
				used_features[29] = True
				features_pair.append(semantic_relations['hyperonyms'][pair])
			if flag == 0:
				used_features[30] = True
				features_pair.append(semantic_relations['other'][pair])
			if flag == 1 and ('n' in pos_tags.columns):
				used_features[31] = True
				features_pair.append(pos_tags['n'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('prop' in pos_tags.columns):
				used_features[32] = True
				features_pair.append(pos_tags['prop'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('adj' in pos_tags.columns):
				used_features[33] = True
				features_pair.append(pos_tags['adj'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('n-adj' in pos_tags.columns):
				used_features[34] = True
				features_pair.append(pos_tags['n-adj'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('v-fin' in pos_tags.columns):
				used_features[35] = True
				features_pair.append(pos_tags['v-fin'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('v-inf' in pos_tags.columns):
				used_features[36] = True
				features_pair.append(pos_tags['v-inf'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('v-pcp' in pos_tags.columns):
				used_features[37] = True
				features_pair.append(pos_tags['v-pcp'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('v-ger' in pos_tags.columns):
				used_features[38] = True
				features_pair.append(pos_tags['v-ger'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('art' in pos_tags.columns):
				used_features[39] = True
				features_pair.append(pos_tags['art'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('pron-pers' in pos_tags.columns):
				used_features[40] = True
				features_pair.append(pos_tags['pron-pers'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('pron-det' in pos_tags.columns):
				used_features[41] = True
				features_pair.append(pos_tags['pron-det'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('pron-indp' in pos_tags.columns):
				used_features[42] = True
				features_pair.append(pos_tags['pron-indp'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('adv' in pos_tags.columns):
				used_features[43] = True
				features_pair.append(pos_tags['adv'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('num' in pos_tags.columns):
				used_features[44] = True
				features_pair.append(pos_tags['num'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('prp' in pos_tags.columns):
				used_features[45] = True
				features_pair.append(pos_tags['prp'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('intj' in pos_tags.columns):
				used_features[46] = True
				features_pair.append(pos_tags['intj'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('conj-s' in pos_tags.columns):
				used_features[47] = True
				features_pair.append(pos_tags['conj-s'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('conj-c' in pos_tags.columns):
				used_features[48] = True
				features_pair.append(pos_tags['conj-c'][pair])
			else:
				features_pair.append(0)
			if flag == 1 and ('punc' in pos_tags.columns):
				used_features[49] = True
				features_pair.append(pos_tags['punc'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('all_ners' in ners.columns):
				used_features[50] = True
				features_pair.append(ners['all_ners'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-ABSTRACCAO' in ners.columns):
				used_features[51] = True
				features_pair.append(ners['B-ABSTRACCAO'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-ACONTECIMENTO' in ners.columns):
				used_features[52] = True
				features_pair.append(ners['B-ACONTECIMENTO'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-COISA' in ners.columns):
				used_features[53] = True
				features_pair.append(ners['B-COISA'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-LOCAL' in ners.columns):
				used_features[54] = True
				features_pair.append(ners['B-LOCAL'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-OBRA' in ners.columns):
				used_features[55] = True
				features_pair.append(ners['B-OBRA'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-ORGANIZACAO' in ners.columns):
				used_features[56] = True
				features_pair.append(ners['B-ORGANIZACAO'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-OUTRO' in ners.columns):
				used_features[57] = True
				features_pair.append(ners['B-OUTRO'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-PESSOA' in ners.columns):
				used_features[58] = True
				features_pair.append(ners['B-PESSOA'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-TEMPO' in ners.columns):
				used_features[59] = True
				features_pair.append(ners['B-TEMPO'][pair])
			else:
				features_pair.append(0)
			if flag == 0 and ('B-VALOR' in ners.columns):
				used_features[60] = True
				features_pair.append(ners['B-VALOR'][pair])
			else:
				features_pair.append(0)
			if flag == 1:
				used_features[61] = True
				features_pair.append(dependencies['dependency_parsing_jc'][pair])
			if flag == 1:
				used_features[62] = True
				features_pair.append(wn_cosine_1_2['NG'][pair])
			if flag == 1:
				used_features[63] = True
				features_pair.append(cn_cosine_1_2_3['NG'][pair])
			if flag == 0:
				used_features[64] = True
				features_pair.append(numberbatch[pair])
			if flag == 0:
				used_features[65] = True
				features_pair.append(numberbatch_tfidf[pair])

			tuple_features_pair = tuple(features_pair)
			features.append(tuple_features_pair)

	if f_selection is not None:
		return np.array(features)
	else:
		return np.array(features), used_features

def debug_data(data, filename):
	""" Function used to debug the corpus state during preprocessing """
	if isinstance(data, pd.DataFrame):
		data_to_print = data.values.tolist()
	else:
		data_to_print = data

	with open(filename, 'w') as f:
		for item in data_to_print:
			f.write("%s\n" % item)

def load_embeddings_models():
	""" Function used to load the word-embedding models """

	# ---LOADING WORD2VEC MODEL---
	model_load_path = os.path.join(ROOT_PATH, 'models', 'word2vec', 'NILC', 'nilc_cbow_s300_300k.txt')
	# model_load_path = os.path.join('models', 'word2vec', 'NILC', 'nilc_skip_s300.txt')
	start_time = time.time()
	print("Started loading the word2vec model")
	word2vec_model = KeyedVectors.load_word2vec_format(model_load_path)
	# word2vec_model = None
	print("Model loaded")
	print("--- %s seconds ---" %(time.time() - start_time))
	print('\a')

	# ---LOADING FASTTEXT MODEL---
	model_path = os.path.join(ROOT_PATH, 'models', 'fastText', 'cc.pt.300_300k.vec')
	start_time = time.time()
	print("Started loading the fasttext model")
	fasttext_model = KeyedVectors.load_word2vec_format(model_path)
	# fasttext_model = None
	print("Model loaded")
	print("--- %s seconds ---" %(time.time() - start_time))
	print('\a')	

	# ---LOADING PT-LKB MODEL---
	model_load_path = os.path.join(ROOT_PATH, 'models', 'ontoPT', 'PT-LKB_embeddings_64', 'ptlkb_64_30_200_p_str.emb')
	# model_load_path = os.path.join('models', 'ontoPT', 'PT-LKB_embeddings_128', 'ptlkb_128_80_10_p_str.emb')
	start_time = time.time()
	print("Started loading the PT-LKB-64 model")
	ptlkb64_model = KeyedVectors.load_word2vec_format(model_load_path)
	# ptlkb64_model = None
	print("Model loaded")
	print("--- %s seconds ---" %(time.time() - start_time))
	print('\a')

	# ---LOADING GLOVE-300 MODEL---
	model_load_path = os.path.join(ROOT_PATH, 'models', 'glove', 'glove_s300_300k.txt')
	# model_load_path = os.path.join('models', 'glove', 'glove_s100.txt')
	start_time = time.time()
	print("Started loading the GLOVE 300 dimensions model")
	glove300_model = KeyedVectors.load_word2vec_format(model_load_path)
	# glove300_model = None
	print("Model loaded")
	print("--- %s seconds ---" %(time.time() - start_time))
	print('\a')

	# ---LOADING NUMBERBATCH MODEL---
	model_load_path = os.path.join(ROOT_PATH, 'models', 'numberbatch', 'numberbatch-17.02_pt_tratado.txt')
	start_time = time.time()
	print("Started loading the NUMBERBATCH dimensions model")
	numberbatch_model = KeyedVectors.load_word2vec_format(model_load_path)
	# numberbatch_model = None
	print("Model loaded")
	print("--- %s seconds ---" %(time.time() - start_time))
	print('\a')

	return word2vec_model, fasttext_model, ptlkb64_model, glove300_model, numberbatch_model

def best_percentile_selector(train_features, test_features, train_similarity_target, test_similarity_target, regressor):
	""" Function used to select the best percentile selector """
	percentile_score = 0
	percentiles = [25, 35, 45, 50, 55, 65, 75]
	# percentiles = [45]
	percentile_selector = None
	percentile_train_features_selected = None
	percentile_test_features_selected = None

	for percentile in percentiles:
		print(percentile)
		temp_percentile_selector = SelectPercentile(score_func=f_regression, percentile=percentile)
		temp_percentile_selector.fit(train_features, train_similarity_target)
		temp_percentile_train_features_selected = temp_percentile_selector.transform(train_features)
		temp_percentile_test_features_selected = temp_percentile_selector.transform(test_features)

		regressor.fit(temp_percentile_train_features_selected, train_similarity_target)

		temp_score = regressor.score(temp_percentile_test_features_selected, test_similarity_target)
		print("The score on the selected features (Percentile Selector): %.3f" % temp_score)

		if temp_score > percentile_score:
			percentile_score = temp_score
			percentile_selector = temp_percentile_selector
			percentile_train_features_selected = temp_percentile_train_features_selected
			percentile_test_features_selected = temp_percentile_test_features_selected

	percentile_mask = percentile_selector.get_support()
	print("This is the percentile mask: ")
	print(percentile_mask)

	return percentile_selector, percentile_score, percentile_train_features_selected, percentile_test_features_selected, percentile_mask

def best_model_based_selector(train_features, test_features, train_similarity_target, test_similarity_target, regressor):
	""" Function used to select the best model based selector """
	model_based_score = 0
	scaling_factors = ["0.25*mean", "0.5*mean", "median", "1.25*mean", "1.5*mean"]
	# scaling_factors = ["0.5*mean", "median"]
	model_based_selector = None
	model_based_train_features_selected = None
	model_based_test_features_selected = None

	for factor in scaling_factors:
		print(factor)
		temp_model_based_selector = SelectFromModel(RandomForestRegressor(n_estimators=100), threshold=factor)
		temp_model_based_selector.fit(train_features, train_similarity_target)
		temp_model_based_train_features_selected = temp_model_based_selector.transform(train_features)
		temp_model_based_test_features_selected = temp_model_based_selector.transform(test_features)

		regressor.fit(temp_model_based_train_features_selected, train_similarity_target)
		
		temp_score = regressor.score(temp_model_based_test_features_selected, test_similarity_target)
		print("The score on the selected features (Model Based Selector): %.3f" % temp_score)

		if temp_score > model_based_score:
			model_based_score = temp_score
			model_based_selector = temp_model_based_selector
			model_based_train_features_selected = temp_model_based_train_features_selected
			model_based_test_features_selected = temp_model_based_test_features_selected

	model_based_mask = model_based_selector.get_support()
	print("This is the model based mask: ")
	print(model_based_mask)

	return model_based_selector, model_based_score, model_based_train_features_selected, model_based_test_features_selected, model_based_mask

def best_iterative_based_selector(train_features, test_features, train_similarity_target, test_similarity_target, regressor):
	""" Function used to select the best iterative based selector """
	iterative_based_score = 0
	# given that all pairs use the same amount of features, the position 0 was arbitrarily selected to compute the number of features being used
	min_number_features = int(0.15*len(train_features[0]))
	max_number_features = int(0.85*len(train_features[0]))

	# min_number_features = 19
	# max_number_features = 20

	iterative_based_selector = None
	iterative_based_train_features_selected = None
	iterative_based_test_features_selected = None

	for i in range(min_number_features, max_number_features):
		print(i)
		temp_iterative_based_selector = RFE(RandomForestRegressor(n_estimators=100), n_features_to_select=i)
		temp_iterative_based_selector.fit(train_features, train_similarity_target)
		temp_iterative_based_train_features_selected = temp_iterative_based_selector.transform(train_features)
		temp_iterative_based_test_features_selected = temp_iterative_based_selector.transform(test_features)

		regressor.fit(temp_iterative_based_train_features_selected, train_similarity_target)

		temp_score = regressor.score(temp_iterative_based_test_features_selected, test_similarity_target)
		print("The score on the selected features (Iterative Based Selector): %.3f" % temp_score)

		if temp_score > iterative_based_score:
			iterative_based_score = temp_score
			iterative_based_selector = temp_iterative_based_selector
			iterative_based_train_features_selected = temp_iterative_based_train_features_selected
			iterative_based_test_features_selected = temp_iterative_based_test_features_selected

	iterative_based_mask = iterative_based_selector.get_support()
	print("This is the iterative based mask: ")
	print(iterative_based_mask)

	return iterative_based_selector, iterative_based_score, iterative_based_train_features_selected, iterative_based_test_features_selected, iterative_based_mask

def rfe_cross_validation(train_features, train_similarity_target, test_features):
	estimator = GradientBoostingRegressor(n_estimators=100)
	rfecv = RFECV(estimator, step=1, cv=10)
	rfecv.fit(train_features, train_similarity_target)

	selected_train_features = rfecv.transform(train_features)
	selected_test_features = rfecv.transform(test_features)

	rfecv_mask = rfecv.get_support()
	print(rfecv_mask)

	return selected_train_features, selected_test_features

def feature_selection(train_features, test_features, train_similarity_target, test_similarity_target, regressor, used_features):
	""" Function used to perform feature selection """
	# percentile selector
	percentile_selector, percentile_score, percentile_train_features_selected, percentile_test_features_selected, percentile_mask = best_percentile_selector(train_features, test_features, train_similarity_target, test_similarity_target, regressor)

	# model based selector
	model_based_selector, model_based_score, model_based_train_features_selected, model_based_test_features_selected, model_based_mask = best_model_based_selector(train_features, test_features, train_similarity_target, test_similarity_target, regressor)

	# iterative based selector
	iterative_based_selector, iterative_based_score, iterative_based_train_features_selected, iterative_based_test_features_selected, iterative_based_mask = best_iterative_based_selector(train_features, test_features, train_similarity_target, test_similarity_target, regressor)

	all_scores = []

	regressor.fit(train_features, train_similarity_target)
	print("The score on all features: %.3f" % regressor.score(test_features, test_similarity_target))
	all_scores.append(regressor.score(test_features, test_similarity_target))

	# show results for the percentile selector
	all_scores.append(percentile_score)

	# show results for the model based selector
	all_scores.append(model_based_score)

	# show results for the iterative based selector
	all_scores.append(iterative_based_score)

	max_value_position = all_scores.index(max(all_scores))

	if max_value_position == 0:
		print("Returning all features!\n")
		return train_features, test_features
	elif max_value_position == 1:
		percentile_mask = build_mask(percentile_mask, used_features)
		mask_save_path = os.path.join('feature_selection_masks', 'percentile_mask.txt')
		debug_data(percentile_mask, mask_save_path)

		print("Returning features selected with the percentile selector!\n")
		return percentile_selector, percentile_train_features_selected, percentile_test_features_selected
	elif max_value_position == 2:
		model_based_mask = build_mask(model_based_mask, used_features)
		mask_save_path = os.path.join('feature_selection_masks', 'model_based_mask.txt')
		debug_data(model_based_mask, mask_save_path)

		print("Returning features selected with the model based selector!\n")
		return model_based_selector, model_based_train_features_selected, model_based_test_features_selected
	else:
		iterative_based_mask = build_mask(iterative_based_mask, used_features)
		mask_save_path = os.path.join('feature_selection_masks', 'iterative_based_mask.txt')
		debug_data(iterative_based_mask, mask_save_path)

		print("Returning features selected with the iterative based selector!\n")
		return iterative_based_selector, iterative_based_train_features_selected, iterative_based_test_features_selected

def build_mask(mask, unused_features_positions):
	""" Function used to complete the mask with unused features not available for feature selection """
	final_mask = mask.tolist()

	for i in range(len(unused_features_positions)):
		if not unused_features_positions[i]:
			final_mask.insert(i, False)

	return final_mask

def aux_best_percentile_selector(train_features, test_features, train_similarity_target, test_similarity_target, regressor, used_features):
	""" Function used to select the best percentile selector """
	percentile_score = 0
	percentiles = [25, 35, 45, 50, 55, 65, 75]
	# percentiles = [45]
	percentile_selector = None
	percentile_train_features_selected = None
	percentile_test_features_selected = None

	for percentile in percentiles:
		print(percentile)
		temp_percentile_selector = SelectPercentile(score_func=f_regression, percentile=percentile)
		temp_percentile_selector.fit(train_features, train_similarity_target)
		temp_percentile_train_features_selected = temp_percentile_selector.transform(train_features)
		temp_percentile_test_features_selected = temp_percentile_selector.transform(test_features)

		regressor.fit(temp_percentile_train_features_selected, train_similarity_target)

		temp_score = regressor.score(temp_percentile_test_features_selected, test_similarity_target)
		print("The score on the selected features (Percentile Selector): %.3f" % temp_score)

		if temp_score > percentile_score:
			percentile_score = temp_score
			percentile_selector = temp_percentile_selector
			percentile_train_features_selected = temp_percentile_train_features_selected
			percentile_test_features_selected = temp_percentile_test_features_selected

	percentile_mask = percentile_selector.get_support()
	print("This is the percentile mask: ")
	print(percentile_mask)

	percentile_mask = build_mask(percentile_mask, used_features)
	mask_save_path = os.path.join('feature_selection_masks', 'assin2_percentile_based_mask.txt')
	debug_data(percentile_mask, mask_save_path)

	return percentile_train_features_selected, percentile_test_features_selected, percentile_selector

def run_feature_extraction(word2vec_model=None, fasttext_model=None, ptlkb64_model=None, glove300_model=None, numberbatch_model=None):
	""" Function used to compute the models """

	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument('test', help='XML file with test data')
	parser.add_argument('output', help='Output tagged XML file')
	args = parser.parse_args()

	"""
	system_mode = 0 -> uses the variant questions with the system
	system_mode = 1 -> uses the PTPT and PTBR train ASSIN collection datasets with the system
	system_mode = 2 -> uses the PTPT and PTBR train and test ASSIN collection datasets with the system
	system_mode = 3 -> uses the Whoosh collection with the system
	system_mode = 4 -> uses ASSIN 1 and ASSIN 2 training collection datasets with the system
	system_mode = 5 -> uses ASSIN 1 training and testing collection and ASSIN 2 training collection datasets with the system

	run_pipeline = 0 -> uses the pre-computed files with the components needed to extract some features
	run_pipeline = 1 -> uses NLPyPort pipeline which avoids having to pre-compute certain components to extract features
	"""

	system_mode = 5
	run_pipeline = 1

	# Flag to indicate if the extracted features should be written to a file (1) or not (0)
	features_to_file_flag = 0

	# extract labels
	train_pairs = []
	train_pairs.extend(read_xml("datasets/assin/assin1/assin-ptpt-train.xml", need_labels=True))
	train_pairs.extend(read_xml("datasets/assin/assin1/assin-ptbr-train.xml", need_labels=True))

	if system_mode == 2 or system_mode == 5:
		train_pairs.extend(read_xml("datasets/assin/assin1/assin-ptpt-test.xml", need_labels=True))
		train_pairs.extend(read_xml("datasets/assin/assin1/assin-ptbr-test.xml", need_labels=True))
	if system_mode == 4 or system_mode == 5:
		train_pairs.extend(read_xml("datasets/assin/assin2/assin2-train-only.xml", need_labels=True))

	train_similarity_target = np.array([pair.similarity for pair in train_pairs])

	# extract training features
	train_corpus = read_corpus(train_pairs)

	# debug_data(train_corpus, "finetune.train.raw")
	# print("Wrote training corpus")

	# preprocessing(text, tokenization=0, rm_stopwords=0, numbers_to_text=0, to_tfidf=0)
	preprocessed_train_corpus = preprocessing(train_corpus, 0, 0, 0, 0)
	train_features, used_train_features = extract_features(run_pipeline, train_corpus, preprocessed_train_corpus, word2vec_mdl=word2vec_model, fasttext_mdl=fasttext_model, ptlkb64_mdl=ptlkb64_model, glove300_mdl=glove300_model, numberbatch_mdl=numberbatch_model)

	# write train features to a .csv file
	if features_to_file_flag == 1:
		write_features_to_csv(train_pairs, train_features, "assin1-train-test-assin2-train-ftrain.csv")

	#############################################################
	test_pairs_dev = read_xml('datasets/assin/assin2/assin2-dev.xml', need_labels=False)

	test_corpus_dev = read_corpus(test_pairs_dev)
	# preprocessing(text, tokenization=0, rm_stopwords=0, numbers_to_text=0, to_tfidf=0)
	preprocessed_test_corpus_dev = preprocessing(test_corpus_dev, 0, 0, 0, 0)
	test_features_dev, used_test_features_dev = extract_features(run_pipeline, test_corpus_dev, preprocessed_test_corpus_dev, word2vec_mdl=word2vec_model, fasttext_mdl=fasttext_model, ptlkb64_mdl=ptlkb64_model, glove300_mdl=glove300_model, numberbatch_mdl=numberbatch_model)

	test_pairs_selection = read_xml('datasets/assin/assin2/assin2-dev.xml', need_labels=True)
	test_similarity_target = np.array([pair.similarity for pair in test_pairs_selection])
	#############################################################

	# extract test features
	# test_pairs = read_xml(args.test, need_labels=False)

	# uncomment next line and comment previous one to compute ASSIN 2 submission results
	test_pairs = read_xml_no_attributes(args.test)

	test_corpus = read_corpus(test_pairs)
	# preprocessing(text, tokenization=0, rm_stopwords=0, numbers_to_text=0, to_tfidf=0)
	preprocessed_test_corpus = preprocessing(test_corpus, 0, 0, 0, 0)
	test_features, used_test_features = extract_features(run_pipeline, test_corpus, preprocessed_test_corpus, word2vec_mdl=word2vec_model, fasttext_mdl=fasttext_model, ptlkb64_mdl=ptlkb64_model, glove300_mdl=glove300_model, numberbatch_mdl=numberbatch_model)

	# write test features to a .csv file
	if features_to_file_flag == 1:
		write_features_to_csv(test_pairs, test_features, "assin1-train-test-assin2-train-ftest.csv")

	# extract test features for feature selection (labels needed in order to perform evaluation)
	# test_pairs_selection = read_xml(args.test, need_labels=True)
	# test_similarity_target = np.array([pair.similarity for pair in test_pairs_selection])

	'''
	Select one type of regressor from scikit-learn. Here is a list with some examples: 
		- GaussianProcessRegressor()
		- DecisionTreeRegressor()
		- LinearRegression()
		- BaggingRegressor(n_estimators=100)
		- AdaBoostRegressor(n_estimators=100)
		- GradientBoostingRegressor()
		- RandomForestRegressor(n_estimators=100)
	'''

	regressor = SVR(gamma='scale', C=10.0, kernel='rbf')

	# ensemble = VotingRegressor(estimators=[('svr', regressor_1), ('gb', regressor_2), ('rf', regressor_3)])

	# params = {'svr__C': [1.0, 10.0, 100.0], 'svr__kernel': ['linear', 'poly', 'rbf', 'sigmoid'], 'rf__n_estimators': [10, 20, 100, 200]}

	# params = {'kernel':('linear', 'poly', 'rbf', 'sigmoid')}

	# regressor = GridSearchCV(regressor_1, params, cv=5)

	use_feature_selection = 0

	if use_feature_selection:
		# selected_selector, selected_train_features, selected_test_features = feature_selection(train_features, test_features_dev, train_similarity_target, test_similarity_target, regressor, used_train_features)
		# selected_train_features, selected_test_features = rfe_cross_validation(train_features, train_similarity_target, test_features)
		selected_train_features, selected_test_features, percentile_selector = aux_best_percentile_selector(train_features, test_features_dev, train_similarity_target, test_similarity_target, regressor, used_train_features)

		test_features_selected = percentile_selector.transform(test_features)
		# test_features_selected = selected_selector.transform(test_features)

		regressor.fit(selected_train_features, train_similarity_target)

		# save model to disk
		model_save_path = os.path.join('trained_models', 'SVR_FS.joblib')
		dump(regressor, model_save_path)

		# apply model to the test dataset
		## this needs to be fixed in order to take advantage of the manual feature selection
		predicted_similarity = regressor.predict(test_features_selected)
		# predicted_similarity = regressor.predict(test_features_selected)
	else:
		regressor.fit(train_features, train_similarity_target)

		# save model to disk
		model_save_path = os.path.join('trained_models', 'SVR_NFS.joblib')
		dump(regressor, model_save_path)

		# apply model to the test dataset
		predicted_similarity = regressor.predict(test_features)

	# write output
	tree = ET.parse(args.test)
	root = tree.getroot()
	for i in range(len(test_pairs)):
		pairs = root[i]
		pairs.set('entailment', "None")
		pairs.set('similarity', str(predicted_similarity[i]))

	tree.write(args.output, 'utf-8')

# if __name__ == '__main__':
# 	system_mode = 4
# 	run_pipeline = 1

# 	# Flag to indicate if the extracted features should be written to a file (1) or not (0)
# 	features_to_file_flag = 0

# 	# extract labels
# 	train_pairs = []
# 	# train_pairs.extend(read_xml("datasets/assin/assin1/assin-ptpt-train.xml", need_labels=True))
# 	# train_pairs.extend(read_xml("datasets/assin/assin1/assin-ptbr-train.xml", need_labels=True))

# 	if system_mode == 2 or system_mode == 5:
# 		# train_pairs.extend(read_xml("assin-ptpt-test.xml", need_labels=True))
# 		train_pairs.extend(read_xml("datasets/assin/assin1/assin-ptbr-test.xml", need_labels=True))
# 	if system_mode == 4 or system_mode == 5:
# 		train_pairs.extend(read_xml("datasets/assin/assin2/assin2-train-only.xml", need_labels=True))

# 	train_similarity_target = np.array([pair.similarity for pair in train_pairs])

# 	# extract training features
# 	train_corpus = tl.read_corpus(train_pairs)

# 	debug_data(train_corpus, "finetune.train.raw")
# 	print("Wrote training corpus")

	# test_pairs = read_xml_no_attributes("datasets/assin/assin2/assin2-blind-test.xml")

	# test_corpus = tl.read_corpus(test_pairs)

	# debug_data(test_corpus, "finetune.test.raw")
	# print("Wrote testing corpus")



from xml.etree import cElementTree as ET

import scripts.tools as tl
from sts_model import STSModel
from scripts.xml_reader import read_xml_no_attributes
from scripts.load_embeddings import load_embeddings_models

# test_file_path = 'datasets/assin/assin1/assin-ptbr-test.xml'
# test_file_path = 'datasets/assin/assin1/assin-ptpt-test.xml'
test_file_path = 'datasets/assin/assin2/assin2-blind-test.xml'

test_pairs = read_xml_no_attributes(test_file_path)
test_corpus = tl.read_corpus(test_pairs)

word2vec_model, fasttext_model, ptlkb64_model, glove300_model, numberbatch_model = load_embeddings_models()

model = STSModel()

model.load_model('model_2206_RFR_ablation_study_master')

test_features = model.extract_multiple_features(corpus=test_corpus, to_store=0, word2vec_mdl=word2vec_model, fasttext_mdl=fasttext_model, ptlkb_mdl=ptlkb64_model, glove_mdl=glove300_model, numberbatch_mdl=numberbatch_model)

similarity = model.predict_similarity(test_features)

# write output
tree = ET.parse(test_file_path)
root = tree.getroot()
for i in range(len(test_pairs)):
    pairs = root[i]
    pairs.set('entailment', "None")
    pairs.set('similarity', str(similarity[i]))

tree.write("assin2.xml", 'utf-8')


import csv
from xml.etree import cElementTree as ET

def combine_similarity_entailment(filename_in, filename_out):
	with open(filename_in) as csv_file:
		csv_reader = csv.reader(csv_file, delimiter=',')
		list_csv_reader = list(csv_reader)

	tree = ET.parse(filename_out)
	root = tree.getroot()
	for i in range(len(list_csv_reader)):
		pairs = root[i]
		pairs.set('entailment', str(list_csv_reader[i][1]))

	tree.write(filename_out, 'utf-8')

if __name__ == '__main__':
	file_in = 'run3_ASSIN2_test_ASAPPpy_entailment.csv'
	file_out = 'assin1_all_assin2_competition.xml'

	combine_similarity_entailment(file_in, file_out)



import os
import pandas as pd
import copy

from sklearn.metrics.pairwise import cosine_similarity
from gensim.parsing.preprocessing import strip_non_alphanum, strip_multiple_whitespaces
from random import randint

from ASAPPpy import ROOT_PATH

def read_faqs_variants():
	""" Function used to read the faqs variants """

	faqs_variants_load_path = os.path.join(ROOT_PATH, 'datasets', 'AIA-BDE_v2.0.txt')

	with open(faqs_variants_load_path) as faqs_file:
		faqs_variants_corpus = faqs_file.read().splitlines()

	faqs_variants_corpus = [line.replace('\t', '') for line in faqs_variants_corpus]

	faqs_file.close()

	faqs_variants_corpus = [line.split(':', 1) for line in faqs_variants_corpus]
	faqs_variants_corpus = [line for line in faqs_variants_corpus if len(line) == 2 and line[1]]
	faqs_variants_corpus = [[line[0], strip_non_alphanum(line[1])] if line[0] != 'R' and line[0] != 'P' else [line[0], line[1]] for line in faqs_variants_corpus]
	faqs_variants_corpus = [[line[0].rstrip(), line[1].rstrip()] if line[0] != 'R' and line[0] != 'P' else [line[0], line[1]] for line in faqs_variants_corpus]
	faqs_variants_corpus = [[line[0], strip_multiple_whitespaces(line[1])] if line[0] != 'R' and line[0] != 'P' else [line[0], line[1]] for line in faqs_variants_corpus]
	faqs_variants_corpus = [[line[0], line[1].lower()] if line[0] != 'R' and line[0] != 'P' else [line[0], line[1]] for line in faqs_variants_corpus]

	OG = []
	VIN = []
	VG1 = []
	VG2 = []
	VUC = []
	VMT = []

	position = 0

	for element in faqs_variants_corpus:
		if element[0] == 'P':
			OG.append([element[1]])
			# the same element is added twice to the list in order to simulate the original question as a variant of its own
			OG[position].extend([element[1]])
			VIN.append([element[1]])
			VG1.append([element[1]])
			VG2.append([element[1]])
			VUC.append([element[1]])
			VMT.append([element[1]])

		if element[0] == 'VIN':
			VIN[position].extend([element[1]])

		if element[0] == 'VG1':
			VG1[position].extend([element[1]])

		if element[0] == 'VG2':
			VG2[position].extend([element[1]])

		if element[0] == 'VUC':
			VUC[position].extend([element[1]])

		if element[0] == 'VMT':
			VMT[position].extend([element[1]])

		if element[0] == 'R':
			OG[position].extend([element[1]])
			VIN[position].extend([element[1]])
			VG1[position].extend([element[1]])
			VG2[position].extend([element[1]])
			VUC[position].extend([element[1]])
			VMT[position].extend([element[1]])

			position += 1

	OG = [element for element in OG if len(element) > 2]
	VIN = [element for element in VIN if len(element) > 2]
	VG1 = [element for element in VG1 if len(element) > 2]
	VG2 = [element for element in VG2 if len(element) > 2]
	VUC = [element for element in VUC if len(element) > 2]
	VMT = [element for element in VMT if len(element) > 2]

	# write cleaned corpus to file
	# final_corpus = []
	# for element in VG1:
	#     for i in range(1, len(element)-1):
	#         final_corpus.extend([element[0], element[i]])

	# for element in VG2:
	#     for i in range(1, len(element)-1):
	#         final_corpus.extend([element[0], element[i]])

	# for element in VIN:
	#     for i in range(1, len(element)-1):
	#         final_corpus.extend([element[0], element[i]])

	# for element in VUC:
	#     for i in range(1, len(element)-1):
	#         final_corpus.extend([element[0], element[i]])

	# for element in VMT:
	#     for i in range(1, len(element)-1):
	#         final_corpus.extend([element[0], element[i]])

	# with open('FAQ_todas_variantes_texto_clean.txt', 'w') as clean_file:
	#     for element in final_corpus:
	#         clean_file.write("%s\n" % element)

	# clean_file.close()

	return OG, VIN, VG1, VG2, VUC, VMT

def read_class_set():
	subtitles_class_path = os.path.join(ROOT_PATH, 'datasets', 'divididossopcomlegendas.txt')
	df = pd.read_csv(subtitles_class_path, sep='§', header=0)

	classes = df.values.tolist()

	class_1 = []
	class_2 = []
	class_3 = []
	class_4 = []

	for sentence in classes:
		if sentence[1] == 1:
			tmp_s = sentence[0].split(":")
			class_1.append(tmp_s[1])
		if sentence[1] == 10:
			tmp_s = sentence[0].split(":")
			class_2.append(tmp_s[1])
		if sentence[1] == 11:
			tmp_s = sentence[0].split(":")
			class_3.append(tmp_s[1])
		# this is the class of subtitles
		if sentence[1] == 15:
			class_4.append(sentence[0])

	return class_1, class_2, class_3

def pre_selection(unselected_phrases, model, position_correct_match):
	""" Function used to perform pre-selection """
	embeddings_unselected_phrases = []
	similaritires = []
	selected_phrases = []
	l_unselected_phrases = copy.deepcopy(unselected_phrases)

	for pair in l_unselected_phrases:
		temp_question = [model[word] for word in pair[0] if word in model.vocab]
		temp_variant = [model[word] for word in pair[1] if word in model.vocab]
		mean_question = sum(temp_question)/len(temp_question)
		mean_variant = sum(temp_variant)/len(temp_variant)

		embeddings_unselected_phrases.append([mean_question, mean_variant])

	for i in range(0, len(embeddings_unselected_phrases)):
		similarity = cosine_similarity([embeddings_unselected_phrases[i][0]], [embeddings_unselected_phrases[i][1]])
		similaritires.append(similarity[0][0])

	# compute the index of the 30 questions with the higher similarity
	selected_indexes = sorted(range(len(similaritires)), key=lambda x: similaritires[x])[-30:]

	# the correct match is added at the begining of the list
	if position_correct_match not in selected_indexes:
		# selected_indexes.insert(0, position_correct_match)
		# selected_phrases.extend(l_unselected_phrases[position_correct_match])

		# for j in range(len(selected_indexes)):
		#     selected_phrases.extend(l_unselected_phrases[selected_indexes[j]])
		selected_phrases = None
		selected_indexes = None
	else:
		selected_phrases.extend(l_unselected_phrases[position_correct_match])

		for j in range(len(selected_indexes)):
			if selected_indexes[j] != position_correct_match:
				selected_phrases.extend(l_unselected_phrases[selected_indexes[j]])
			else:
				selected_indexes.insert(0, selected_indexes.pop(j))

	return selected_phrases, selected_indexes

def n_max_elements(list1, N):
	""" Function to compute the N highest numbers of a list """
	n_list1 = copy.deepcopy(list1)
	final_list = []

	for i in range(0, N):
		max1 = 0

		for j in range(len(n_list1)):
			if n_list1[j] > max1:
				max1 = n_list1[j]

		n_list1.remove(max1)
		final_list.append(max1)

	return final_list

def n_max_elements_indexes(list1, N):
	""" Function to compute the N highest numbers of a list """
	# Note: Not the most efficient way of doing it

	n_list1 = copy.deepcopy(list1)
	elem_pos_list = []
	indexes_list = []

	for k in range(len(n_list1)):
		elem_pos_list.append([n_list1[k], k])

	for i in range(0, N):
		max_element = -1
		max_position = -1
		max_tmp_j = -1

		for j in range(len(elem_pos_list)):
			if elem_pos_list[j][0] > max_element:
				max_element = elem_pos_list[j][0]
				max_position = elem_pos_list[j][1]
				max_tmp_j = j

		indexes_list.append(max_position)
		elem_pos_list.remove(elem_pos_list[max_tmp_j])

	indexes_list.sort()

	return indexes_list

def select_multiple_random_subtitles():
	subtle_path = os.path.join(ROOT_PATH, 'dataset', 'SubtleCorpusPTEN', 'por', 'corpus0sDialogues_clean.txt')
	selected_interactions = []
	selected_positions = []

	with open(subtle_path) as suble_file:
		subtle_corpus = suble_file.read().splitlines()

	suble_file.close()

	for i in range(0, 379):
		position = randint(0, len(subtle_corpus))

		if position not in selected_positions:
			# if len(subtle_corpus[position]) >= 4:
			selected_interactions.append(subtle_corpus[position])
			selected_positions.append(position)

	return selected_interactions

def write_output_file(model_path, accuracies, classif_flag, fselect_flag, preselect_flag):

	if '/' in model_path:
		split_model_path = model_path.split('/')
		model_name = split_model_path[1].split('.')

		filename = '{}_classif_{}_fselect_{}_preselect_{}.txt'.format(model_name[0], classif_flag, fselect_flag, preselect_flag)
	else:
		filename = '{}_classif_{}_fselect_{}_preselect_{}.txt'.format(model_path, classif_flag, fselect_flag, preselect_flag)

	filename_path = 'computed_accuracies/{}'.format(filename)

	if os.path.exists(filename_path):
		tmp_filename = input("The designated file already exists. Please insert a new filename: ")

		filename_path = 'computed_accuracies/{}.txt'.format(tmp_filename)

		while os.path.exists(filename_path):
			tmp_filename = input("The designated file already exists. Please insert a new filename: ")

			filename_path = 'computed_accuracies/{}.txt'.format(tmp_filename)

		with open(filename_path, 'w') as accuracies_file:
			for item in accuracies:
				accuracies_file.write("%s\n" % item)

		accuracies_file.close()

	else:
		with open(filename_path, 'w') as accuracies_file:
			for item in accuracies:
				accuracies_file.write("%s\n" % item)

		accuracies_file.close()



#! /usr/bin/env python
#
# Copyright (c) 2020 José Santos
# License: MIT License
from setuptools import setup, find_packages
from setuptools.command.install import install

import ASAPPpy
# from download_models import download_from_google_drive

# class DownloadModels(install):
#     """Customized setuptools install command - Downloads STS models from Google Drive."""

#     def run(self):
#         install.run(self)
#         download_from_google_drive('1VmCflR_ULeyQx38u6kKIVgoMBAtCc4jd', 'ASAPPpy')

with open('README.md') as f:
    LONG_DESCRIPTION = f.read()

setup(
    name='ASAPPpy',
    version=ASAPPpy.__version__,
    description='Semantic Textual Similarity and Dialogue System package for Python',
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    
    packages=find_packages(),
    include_package_data=True,
    # cmdclass={
    #     'install': DownloadModels,
    # },

    author=u'José Santos',
    author_email='santos@student.dei.uc.pt',

    license='MIT License',

    keywords='Natural Language Processing, NLP,'
        'Sentence Similarity, Semantic Textual Similarity, STS,'
        'Dialogue Agents, Chatbot Framework, Chatbot',

    platforms='any',

    zip_safe=False,

    classifiers=[  # from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Text Processing :: Linguistic',
    ],

    python_requires='>=3.6.1',
    install_requires=[
        'setuptools == 49.6.0',
        'scikit-learn == 0.22.2',
        'pandas >= 1.1.1',
        'requests',
        'slackclient == 2.1.0',
        'slackeventsapi == 2.1.0',
        'nltk',
        'NLPyPort',
        'spacy',
        'gensim',
        'joblib',
        'num2words',
        'Whoosh',
        'Keras',
        'tensorflow',
        'cufflinks',
        'matplotlib',
        'seaborn',
        'imblearn',
    ],
)


import sys
import os
from whoosh.qparser import QueryParser, OrGroup
from whoosh import scoring
from whoosh.index import open_dir
from whoosh.query import FuzzyTerm
from gensim.parsing.preprocessing import strip_non_alphanum
from gensim.parsing.preprocessing import strip_multiple_whitespaces

def query_indexer(query_string, directory, topN=30):
	'''
	query_string - sentence used to perform the search.
	directory - location of the indexer to be used.
	topN - number of documents returned by the query. The default is 30.
	'''
	ix = open_dir(directory)

	query_string = strip_non_alphanum(query_string)
	query_string = strip_multiple_whitespaces(query_string)

	with ix.searcher(weighting=scoring.BM25F) as searcher:
	# with ix.searcher(weighting=scoring.Frequency) as searcher:
		query = QueryParser("question", ix.schema, termclass=FuzzyTerm, group=OrGroup).parse(query_string)
		try:
			options = []
			options_answers = []
			options_docnumbers = []

			loop_range = 0

			results = searcher.search(query, limit=topN, terms=True)

			if topN <= len(results):
				loop_range = topN
			else:
				loop_range = len(results)

			for i in range(loop_range):
				# this needs to be adapted in order to work with the Whoosh Chatbot; uncomment next line in order to work with the normal Chatbot
				options_answers.append(results[i]['response'])
				options.append(results[i]['question'])
				options_docnumbers.append(results[i].docnum)

			return options, options_answers, options_docnumbers

			# return the element with the highest similarity score from the indexer
			# return results[0]['response']
		except IndexError:
			return None

# if __name__ == '__main__':
# 	question = "ola, tudo bem?"
# 	question = "sou o Jose"
# 	responses = query_indexer(question)
# 	print(responses)



import keras as k
import pandas as pd
import cufflinks as cf
from sklearn.model_selection import train_test_split
from keras import Sequential
from keras.layers import  Embedding
from keras.layers import Dense,SpatialDropout1D
from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences
from keras.layers.recurrent import LSTM
import numpy as np 
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from keras.preprocessing.text import Tokenizer
from keras.preprocessing.sequence import pad_sequences
from keras.models import Sequential
from keras.layers import Dense, Embedding, LSTM, SpatialDropout1D
from sklearn.model_selection import train_test_split
from keras.utils.np_utils import to_categorical
from keras.callbacks import EarlyStopping
from keras.layers import Dropout
from keras.models import Model, load_model
from sklearn.utils import resample
import tensorflow as tf
from keras.optimizers import Adam, RMSprop
from imblearn.over_sampling import SMOTE
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.naive_bayes import MultinomialNB
from sklearn.metrics import classification_report
from sklearn.naive_bayes import GaussianNB
import pickle
from sklearn import svm
from sklearn.feature_extraction.text import TfidfVectorizer
#naive bayes, svm, random forest



#https://github.com/susanli2016/NLP-with-Python/blob/master/Multi-Class%20Text%20Classification%20LSTM%20Consumer%20complaints.ipynb

#1---Empresa na Hora
#2---Marca na hora e marca na hora online
#3---Formas jurídicas
#4---Cartão de Empresa/Cartão de Pessoa Coletiva
#5---Criação da Empresa Online
#6---Certificados de Admissibilidade
#7---Inscrições Online
#8---Certidões Online
#9---Gestão da Empresa Online
#10---RJACSR
#11---Alojamento Local

def escala(X,y):
	from imblearn.over_sampling import RandomOverSampler
	sampler = RandomOverSampler(sampling_strategy='not majority',random_state=0)
	X_train, Y_train = sampler.fit_sample(X, y)
	return X_train,Y_train

def treina(model_name):

	df = pd.read_csv("dataset/divididossopcomlegendas.txt",sep='§',header=0)
	df.info()

	max_len = 0
	for value in df.Perguntas:
		if(len(value)>max_len):
			#print(value)
			max_len = len(value)
	max_words = 0
	for value in df.Perguntas:
		word_count = len(value.split(" "))
		if(word_count>max_words):
			#print(word_count)
			max_words = word_count
	#print("---------")
	#print(max_words)
	#print(df.Class.value_counts().sort_values())
	g = df.groupby('Class')
	g= pd.DataFrame(g.apply(lambda x: x.sample(g.size().min()).reset_index(drop=True)))
	print(df)

	#df =  pd.DataFrame(g.apply(lambda x: x.sample(g.size().min()).reset_index(drop=True)))
	#print("###")
	#print(df.shape)
	#print(df.head(10))
	#df = g



	#X_conveted = pd.get_dummies(df["Perguntas"])
	#Divisão em treino e teste
	X_train, X_test, Y_train, Y_test = train_test_split(df["Perguntas"],df["Class"], test_size = 0.3, random_state = 42)
	#print(X_train.shape)
	#print(Y_train.shape)
	#print(X_test.shape)
	#print(Y_test.shape)



	vect = TfidfVectorizer().fit(X_train)

	with open("vect", 'wb') as fid:
		pickle.dump(vect, fid)	

	X_train_vectorized = vect.transform(X_train)

	X_train_vectorized, Y_train = escala(X_train_vectorized, Y_train)

	clf = svm.SVC(gamma="scale",kernel='linear', degree=16)
	clf.fit(X_train_vectorized,Y_train)

	with open(model_name, 'wb') as fid:
		pickle.dump(clf, fid)	

	preds = clf.predict(vect.transform(X_test))
	score = classification_report(Y_test, preds)
	print(score)

	return vect


def corre(modelo,v):
	labels = ["Empresa na Hora", "Marca na hora e marca na hora online", "Formas jurídicas", "Cartão de Empresa/Cartão de Pessoa Coletiva", "Criação da Empresa Online","Certificados de Admissibilidade", "Inscrições Online","Certidões Online", "Gestão da Empresa Online","RJACSR","Alojamento Local", "Fora do Domínio", "Fora do Domínio", "Fora do Domínio", "Fora do Domínio", "Fora do Domínio"]

	with open(modelo, 'rb') as fid:
		clfrNB = pickle.load(fid)

	with open("vect", 'rb') as fid:
		v = pickle.load(fid)
	a = 0

	while (a==0):
		print("Entrada.")
		entrada = input()
		entrada = [entrada]
		#entrada = pd.DataFrame([entrada])
		print("Entrada recebida.")
		transformada = v.transform(entrada)
		preds = clfrNB.predict(transformada)
		print(preds[0])
		print(labels[preds[0]])

def svm_classifier(frase):

	labels = ["Empresa na Hora", 1, "Formas jurídicas", "Cartão de Empresa/Cartão de Pessoa Coletiva", "Criação da Empresa Online", "Certificados de Admissibilidade", "Inscrições Online", "Certidões Online", "Gestão da Empresa Online", 2, 3, "Fora do Domínio", "Fora do Domínio", "Fora do Domínio", "Fora do Domínio", 4]

	with open("trained_models/svm_linear_kernel.pickle", 'rb') as fid:
		clfrNB = pickle.load(fid)

	with open("vectors/vect", 'rb') as fid:
		v = pickle.load(fid)

	entrada = [frase]
	transformada = v.transform(entrada)
	preds = clfrNB.predict(transformada)

	return labels[preds[0]]


if __name__ == '__main__':
	#_lr_0.03
	#modelo = "lstm_com_balanceamento_varias_camadas_500_lr_0.03.h5"
	#modelo = "lstm_com_balanceamento_varias_camadas_200_lr_0.03.h5"
	# vect = treina("svm_linear_kernel.pickle")
	# corre("svm_linear_kernel.pickle", "")
	svm_classifier("ola")







'''
Module with auxiliary functions.
'''

from .xml_reader import read_xml

from gensim.parsing.preprocessing import strip_non_alphanum
from gensim.parsing.preprocessing import strip_multiple_whitespaces
from gensim.matutils import Sparse2Corpus
from string import punctuation
from nltk.corpus import stopwords
import pandas as pd
from num2words import num2words
from sklearn.feature_extraction.text import TfidfVectorizer

from sklearn.metrics.pairwise import cosine_similarity

def build_sentences_from_tokens(tokens):
	""" Function used to rebuild the sentences from the tokens returned by the pipeline """

	sentences = []
	tmp_sentence = []

	for elem in tokens:
		if elem == "EOS":
			tmp_sentence = ' '.join(tmp_sentence)
			sentences.append(tmp_sentence)
			tmp_sentence = []
		else:
			tmp_sentence.append(elem)

	return sentences

def write_data_to_file(data, filename):
	""" Function used to debug the corpus state during preprocessing """
	if isinstance(data, pd.DataFrame):
		data_to_print = data.values.tolist()
	else:
		data_to_print = data

	with open(filename, 'w') as f:
		for item in data_to_print:
			f.write("%s\n" % item)

def write_features_to_csv(pairs, features, filename):
	""" Function used to write the features dataframe to a .csv file. """
	ids = []

	for pair in pairs:
		ids.append(pair.id)

	features_dataframe = pd.DataFrame(features)
	features_dataframe.insert(0, column="ID", value=ids)
	features_dataframe.to_csv(filename, index=False)

def deprecated_read_corpus(filename):
	""" Function used to read the corpus for training (old version). """
	train_list = read_xml(filename, 1)

	corpus = []

	for pair in train_list:
		corpus += [strip_multiple_whitespaces(pair.t)]
		corpus += [strip_multiple_whitespaces(pair.h)]

	return corpus

def read_corpus(pairs):
	""" Function used to read the corpus for training. """

	corpus = []

	for pair in pairs:
		corpus += [strip_multiple_whitespaces(pair.t)]
		corpus += [strip_multiple_whitespaces(pair.h)]

	return corpus

#pre-process the data
def preprocessing(text, tokenization=0, rm_stopwords=0, numbers_to_text=0, to_tfidf=0):
	""" Function used to preprocess the training data """
	train_data = pd.DataFrame(columns=['text', 'response'])

	prep_0 = [strip_non_alphanum(line) for line in text]
	prep_1 = [line for line in prep_0 if line.rstrip()]
	prep_2 = [strip_multiple_whitespaces(line) for line in prep_1]
	prep_3 = [line.lower() for line in prep_2]

	if to_tfidf == 1:
		#when using tf_idf, removes single character words given that they are ignored by sklearn's TfidfVectorizer
		prep_3 = [' '.join([word for word in line.split() if len(word) > 1]) for line in prep_3]

	if tokenization == 1:
		prep_3 = [line.split(' ') for line in prep_3]
		#removes whitespaces from the list
		prep_3 = [list(filter(None, line)) for line in prep_3]
	else:
		prep_3 = [line[:-1] if line[-1] == " " else line for line in prep_3]

	if numbers_to_text == 1 and tokenization == 1:
		#convert all numbers to integers and convert these numbers to its written form
		temp_prep = []
		for sentence in prep_3:
			temporary_sentence = []
			for word in sentence:
				if str(word).isdigit():
					converted_words = num2words(int(word), to='cardinal', lang='pt').split(' ')
					if to_tfidf == 1 and rm_stopwords == 0:
						converted_words = [word for word in converted_words if word != 'e']
					temporary_sentence.extend(converted_words)
				else:
					temporary_sentence.append(word)
			temp_prep.append(temporary_sentence)

		prep_3 = temp_prep
	elif numbers_to_text == 1 and tokenization == 0:
		#convert all numbers to integers and convert these numbers to its written form
		temp_prep = []
		for sentence in prep_3:
			temporary_sentence = []
			for word in sentence.split(' '):
				if str(word).isdigit():
					converted_words = num2words(int(word), to='cardinal', lang='pt').split(' ')
					if to_tfidf == 1 and rm_stopwords == 0:
						converted_words = [word for word in converted_words if word != 'e']
					temporary_sentence.extend(converted_words)
				else:
					temporary_sentence.append(word)
			temporary_sentence = ' '.join(temporary_sentence)
			temp_prep.append(temporary_sentence)
		prep_3 = temp_prep

	if rm_stopwords == 1:
		stp = set(stopwords.words('portuguese') + list(punctuation))
		if tokenization == 1:
			prep_3 = [[word for word in sentence if word not in stp] for sentence in prep_3]
		elif tokenization == 0:
			prep_3 = [' '.join([word for word in sentence.split(' ') if word not in stp]) for sentence in prep_3]

	tmp = pd.DataFrame({'text':prep_3[::2], 'response':prep_3[1::2]})
	train_data = train_data.append(tmp[['text', 'response']], ignore_index=True)

	return train_data

def compute_tfidf_matrix(text, remove_stopwords=0, to_gensim=0, compute_cos_similarity=0):
	""" Function used to compute the TF-IDF Matrix of the corpus """
	stp = set(stopwords.words('portuguese') + list(punctuation))

	list_1 = list(text['text'])
	list_2 = list(text['response'])

	corpus = list_1 + list_2
	corpus[::2] = list_1
	corpus[1::2] = list_2

	if remove_stopwords == 0:
		vectorizer = TfidfVectorizer(stop_words=None, min_df=0, max_features=None)
	else:
		vectorizer = TfidfVectorizer(stop_words=stp, min_df=0, max_features=None)

	tfidf_matrix = vectorizer.fit_transform(corpus)

	feature_names = vectorizer.get_feature_names()

	#word2tfidf = dict(zip(vectorizer.get_feature_names(), vectorizer.idf_))

	#for word, score in word2tfidf.items():
		#print(word, score)

	if to_gensim:
		tfidf_matrix_list = list(Sparse2Corpus(tfidf_matrix, documents_columns=False))

		#convert gensim format to list
		temp_tfidf_matrix_list = []
		for sentence in tfidf_matrix_list:

			temp_sentence = []
			for word in sentence:
				temp_word = feature_names[word[0]]
				temp_pair = (temp_word, word[1])

				temp_sentence.append(temp_pair)
			temp_tfidf_matrix_list.append(dict(temp_sentence))

		tfidf_matrix_list = temp_tfidf_matrix_list

		return tfidf_matrix_list

	if compute_cos_similarity:
		tfidf_matrix_list = list(Sparse2Corpus(tfidf_matrix, documents_columns=False))

		#convert gensim format to list
		temp_tfidf_matrix_list = []
		for sentence in tfidf_matrix_list:

			temp_sentence = []
			for word in sentence:
				temp_pair = word[1]

				temp_sentence.append(temp_pair)
			temp_tfidf_matrix_list.append(temp_sentence)

		tfidf_matrix_list = temp_tfidf_matrix_list

		similarity_m = []

		for i in range(0, len(tfidf_matrix_list), 2):
			if len(tfidf_matrix_list[i]) > len(tfidf_matrix_list[i+1]):
				difference_in_length = len(tfidf_matrix_list[i]) - len(tfidf_matrix_list[i+1])
				tfidf_matrix_list[i+1].extend([0] * difference_in_length)
			elif len(tfidf_matrix_list[i]) < len(tfidf_matrix_list[i+1]):
				difference_in_length = len(tfidf_matrix_list[i+1]) - len(tfidf_matrix_list[i])
				tfidf_matrix_list[i].extend([0] * difference_in_length)

			similarity = cosine_similarity([tfidf_matrix_list[i]], [tfidf_matrix_list[i+1]])
			similarity_m.append(similarity[0][0])

		return similarity_m

	return tfidf_matrix

# if __name__ == "__main__":
# 	corpus = deprecated_read_corpus("assin-ptpt-test.xml")

# 	#function arguments
# 	remove_stopwords = 0
# 	convert_num_to_text = 0
# 	tf_idf = 1
# 	# when it comes to word2vec, the preprocessing function should always have tokenization equal to 1
# 	preprocessed_corpus = preprocessing(corpus, 0, remove_stopwords, convert_num_to_text, tf_idf)

# 	print(preprocessed_corpus)

# 	matrix = compute_tfidf_matrix(preprocessed_corpus, remove_stopwords, 0)
# 	tfidf_matrix_list = list(Sparse2Corpus(matrix, documents_columns=False))

# 	#convert gensim format to list
# 	temp_tfidf_matrix_list = []
# 	for sentence in tfidf_matrix_list:

# 		temp_sentence = []
# 		for word in sentence:
# 			temp_pair = word[1]

# 			temp_sentence.append(temp_pair)
# 		temp_tfidf_matrix_list.append(temp_sentence)

# 	tfidf_matrix_list = temp_tfidf_matrix_list

# 	similarity_m = []
# 	sized = int(len(tfidf_matrix_list)/2)

# 	for i in range(0, sized, 2):
# 		if len(tfidf_matrix_list[i]) > len(tfidf_matrix_list[i+1]):
# 			difference_in_length = len(tfidf_matrix_list[i]) - len(tfidf_matrix_list[i+1])
# 			tfidf_matrix_list[i+1].extend([0] * difference_in_length)
# 		elif len(tfidf_matrix_list[i]) < len(tfidf_matrix_list[i+1]):
# 			difference_in_length = len(tfidf_matrix_list[i+1]) - len(tfidf_matrix_list[i])
# 			tfidf_matrix_list[i].extend([0] * difference_in_length)

# 		similarity = cosine_similarity([tfidf_matrix_list[i]], [tfidf_matrix_list[i+1]])
# 		similarity_m.append(similarity[0][0])



'''
Module used to extract lexical features from a given corpus.
'''

import copy

from nltk import ngrams
from sklearn.metrics.pairwise import cosine_similarity

def aux_create_character_ngrams(sentence, number_of_grams):
	""" Auxiliar function used to create character ngrams, in order for them to be restricted to the boundaries of the word """
	allgrams = []
	for word in sentence.split():
		word_ngrams = list(ngrams(word, number_of_grams))

		for gram in word_ngrams:
			joint_gram = ''.join(gram)
			allgrams.append(joint_gram)
	return allgrams

def create_character_ngrams(original_text, number_of_grams):
	""" Function used to create character ngrams """
	text = copy.deepcopy(original_text)
	#during the preprocessing step, tokenization should not be applied

	#remove all words that are smaller than the least number of characters the n-gram will have
	text['text'] = text['text'].apply(lambda x: ' '.join([word for word in x.split() if len(word) >= number_of_grams]))
	text['response'] = text['response'].apply(lambda x: ' '.join([word for word in x.split() if len(word) >= number_of_grams]))

	#produces the character ngrams
	text['text'] = text['text'].apply(lambda x: aux_create_character_ngrams(x, number_of_grams))
	text['response'] = text['response'].apply(lambda x: aux_create_character_ngrams(x, number_of_grams))

	#Manual alternative, it could be useful coming back to it if something goes wrong with the current implementation
	#text['text'] = text['text'].apply(lambda x: [x[i:i+number_of_grams] for i in range(len(x)-number_of_grams+1)])
	#text['response'] = text['response'].apply(lambda x: [x[i:i+number_of_grams] for i in range(len(x)-number_of_grams+1)])

	return text

def aux_create_multiple_character_ngrams(sentence, number_of_grams):
	""" Auxiliar function used to create the union of multiple character ngrams of a sentence """
	all_grams = []

	for number in number_of_grams:
		temp_grams = []

		for word in sentence.split():
			if len(word) >= number:
				word_ngrams = list(ngrams(word, number))

				for gram in word_ngrams:
					joint_gram = ''.join(gram)
					temp_grams.append(joint_gram)
		all_grams.extend(temp_grams)

	return all_grams

def create_multiple_character_ngrams(original_text, *number_of_grams):
	""" Function used to create the union of multiple character ngrams of a sentence """
	text = copy.deepcopy(original_text)

	#produces the character ngrams
	text['text'] = text['text'].apply(lambda x: aux_create_multiple_character_ngrams(x, number_of_grams))
	text['response'] = text['response'].apply(lambda x: aux_create_multiple_character_ngrams(x, number_of_grams))

	return text

def create_word_ngrams(original_text, number_of_grams):
	""" Function used to create word ngrams """
	text = copy.deepcopy(original_text)
	#during the preprocessing step, tokenization should not be applied
	text['text'] = text['text'].apply(lambda x: list(ngrams(x.split(), number_of_grams)))
	text['response'] = text['response'].apply(lambda x: list(ngrams(x.split(), number_of_grams)))

	return text

def aux_create_multiple_word_ngrams(sentence, number_of_grams):
	""" Auxiliar function used to create the union of multiple word ngrams of a sentence """
	all_grams = []
	string_all_grams = []
	splitted_sentence = sentence.split()

	for number in number_of_grams:
		all_grams.extend(list(ngrams(splitted_sentence, number)))

	for gram in all_grams:
		temp_string = ' '.join(gram)
		string_all_grams.append(temp_string)

	return string_all_grams

def create_multiple_word_ngrams(original_text, *number_of_grams):
	""" Function used to create the union of multiple word ngrams of a sentence """
	text = copy.deepcopy(original_text)
	#during the preprocessing step, tokenization should not be applied
	text['text'] = text['text'].apply(lambda x: aux_create_multiple_word_ngrams(x, number_of_grams))
	text['response'] = text['response'].apply(lambda x: aux_create_multiple_word_ngrams(x, number_of_grams))

	return text

def jaccard_coefficient(label1, label2):
	""" Distance metric comparing set-similarity. """
	#the original formula used float, but it doesn't seem to make any difference
	#return float(len(label1 & label2)) / min(len(label1), len(label2))
	if len(label1.union(label2)) == 0:
		return 0
	else:
		return len(label1.intersection(label2)) / len(label1.union(label2))

def compute_jaccard(text):
	""" Function used to compute the Jaccard distance between two sentences in the corpus """
	#print(text)
	#during the preprocessing step, tokenization should be applied
	text['jaccard'] = text.apply(lambda pair: jaccard_coefficient(set(pair['text']), set(pair['response'])), axis=1)
	return text

def overlap_coefficient(label1, label2):
	""" Distance metric comparing set-similarity. """
	#the original formula used float, but it doesn't seem to make any difference
	#return float(len(label1 & label2)) / min(len(label1), len(label2))
	if min(len(label1), len(label2)) == 0:
		return 0
	else:
		return len(label1.intersection(label2)) / min(len(label1), len(label2))

def compute_overlap(text):
	#print(text)
	""" Function used to compute the Overlap coefficient between two sentences in the corpus """
	text['overlap'] = text.apply(lambda pair: overlap_coefficient(set(pair['text']), set(pair['response'])), axis=1)
	return text

def dice_distance(label1, label2):
	""" Distance metric comparing set-similarity. """
	if (len(label1) + len(label2)) == 0:
		return 0
	else:
		return len(label1.intersection(label2)) / (len(label1) + len(label2))
    #return (len(label1.union(label2)) - len(label1.intersection(label2))) / len(label1.union(label2))

def compute_dice(text):
	""" Function used to compute the Dice coefficient between two sentences in the corpus """
	text['dice'] = text.apply(lambda pair: dice_distance(set(pair['text']), set(pair['response'])), axis=1)
	return text

def ng_cosine(label1, label2):
	""" Cosine similarity between pairs of sentences. """
	pair_gram_union = label1.union(label2)

	label1_vector = []
	label2_vector = []

	if len(pair_gram_union) == 0:
		return 0
	else:
		for gram in pair_gram_union:
			if gram in label1:
				label1_vector.append(1)
			else:
				label1_vector.append(0)

			if gram in label2:
				label2_vector.append(1)
			else:
				label2_vector.append(0)

		similarity = cosine_similarity([label1_vector], [label2_vector])

		return similarity[0][0]

def NG(text):
	""" Function used to compute the Cosine Similarity between two sentences in the corpus """
	text['NG'] = text.apply(lambda pair: ng_cosine(set(pair['text']), set(pair['response'])), axis=1)
	return text
